from telethon.tl.mtproto_request import MTProtoRequest


class DocumentAttributeAudio(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    documentAttributeAudio#9852f9c6 flags:None voice:flags.10?true duration:int title:flags.0?string performer:flags.1?string waveform:flags.2?bytes = DocumentAttribute"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9852f9c6

    def __init__(self, duration, voice=None, title=None, performer=None, waveform=None):
        """
        :param voice: Telegram type: «true».
        :param duration: Telegram type: «int».
        :param title: Telegram type: «string».
        :param performer: Telegram type: «string».
        :param waveform: Telegram type: «bytes».
        """
        super().__init__()

        self.voice = voice
        self.duration = duration
        self.title = title
        self.performer = performer
        self.waveform = waveform

    def on_send(self, writer):
        writer.write_int(DocumentAttributeAudio.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 10) if self.voice else 0
        flags |= (1 << 0) if self.title else 0
        flags |= (1 << 1) if self.performer else 0
        flags |= (1 << 2) if self.waveform else 0
        writer.write_int(flags)

        writer.write_int(self.duration)
        if self.title:
            writer.tgwrite_string(self.title)

        if self.performer:
            writer.tgwrite_string(self.performer)

        if self.waveform:
            writer.tgwrite_bytes(self.waveform)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DocumentAttributeAudio(None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 10)) != 0:
            self.voice = True  # Arbitrary not-None value, no need to read since it is a flag

        self.duration = reader.read_int()
        if (flags & (1 << 0)) != 0:
            self.title = reader.tgread_string()

        if (flags & (1 << 1)) != 0:
            self.performer = reader.tgread_string()

        if (flags & (1 << 2)) != 0:
            self.waveform = reader.tgread_bytes()

    def __repr__(self):
        return 'documentAttributeAudio#9852f9c6 flags:None voice:flags.10?true duration:int title:flags.0?string performer:flags.1?string waveform:flags.2?bytes = DocumentAttribute'

    def __str__(self):
        return '(documentAttributeAudio (ID: 0x9852f9c6) = (voice={}, duration={}, title={}, performer={}, waveform={}))'.format(str(self.voice), str(self.duration), str(self.title), str(self.performer), str(self.waveform))



from telethon.tl.mtproto_request import MTProtoRequest


class Link(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.link#3ace484c my_link:ContactLink foreign_link:ContactLink user:User = contacts.Link"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3ace484c

    def __init__(self, my_link, foreign_link, user):
        """
        :param my_link: Telegram type: «ContactLink».
        :param foreign_link: Telegram type: «ContactLink».
        :param user: Telegram type: «User».
        """
        super().__init__()

        self.my_link = my_link
        self.foreign_link = foreign_link
        self.user = user

    def on_send(self, writer):
        writer.write_int(Link.constructor_id, signed=False)
        self.my_link.on_send(writer)
        self.foreign_link.on_send(writer)
        self.user.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Link(None, None, None)

    def on_response(self, reader):
        self.my_link = reader.tgread_object()
        self.foreign_link = reader.tgread_object()
        self.user = reader.tgread_object()

    def __repr__(self):
        return 'contacts.link#3ace484c my_link:ContactLink foreign_link:ContactLink user:User = contacts.Link'

    def __str__(self):
        return '(contacts.link (ID: 0x3ace484c) = (my_link={}, foreign_link={}, user={}))'.format(str(self.my_link), str(self.foreign_link), str(self.user))



from telethon.tl.types import MessageEntityBold, MessageEntityItalic, MessageEntityCode, MessageEntityTextUrl


def parse_message_entities(msg):
    """Parses a message and returns the parsed message and the entities (bold, italic...).
       Note that although markdown-like syntax is used, this does not reflect the complete specification!"""

    # Store the entities here
    entities = []

    # Convert the message to a mutable list
    msg = list(msg)

    # First, let's handle all the text links in the message, so afterwards it's clean
    # for us to get our hands dirty with the other indicators (bold, italic and fixed)
    url_indices = [None] * 4  # start/end text index, start/end url index
    valid_url_indices = []  # all the valid url_indices found
    for i, c in enumerate(msg):
        if c is '[':
            url_indices[0] = i

        # From now on, also ensure that the last item was set
        elif c == ']' and url_indices[0]:
            url_indices[1] = i

        elif c == '(' and url_indices[1]:
            # If the previous index (']') is not exactly before the current index ('('),
            # then it's not a valid text link, so clear the previous state
            if url_indices[1] != i - 1:
                url_indices[:2] = [None] * 2
            else:
                url_indices[2] = i

        elif c == ')' and url_indices[2]:
            # We have succeeded to find a markdown-like text link!
            url_indices[3] = i
            valid_url_indices.append(url_indices[:])  # Append a copy
            url_indices = [None] * 4

    # Iterate in reverse order to clean the text from the urls
    # (not to affect previous indices) and append MessageEntityTextUrl's
    for i in range(len(valid_url_indices) - 1, -1, -1):
        vui = valid_url_indices[i]

        # Add 1 when slicing the message not to include the [] nor ()
        # There is no need to subtract 1 on the later part because that index is already excluded
        link_text = ''.join(msg[vui[0]+1:vui[1]])
        link_url = ''.join(msg[vui[2]+1:vui[3]])

        # After we have retrieved both the link text and url, replace them in the message
        # Now we do have to add 1 to include the [] and () when deleting and replacing!
        del msg[vui[2]:vui[3]+1]
        msg[vui[0]:vui[1]+1] = link_text

        # Finally, update the current valid index url to reflect that all the previous VUI's will be removed
        # This is because, after the previous VUI's get done, their part of the message is removed too,
        # hence we need to update the current VUI subtracting that removed part length
        for prev_vui in valid_url_indices[:i]:
            prev_vui_length = prev_vui[3] - prev_vui[2] - 1
            displacement = prev_vui_length + len('[]()')
            vui[0] -= displacement
            vui[1] -= displacement
            # No need to subtract the displacement from the URL part (indices 2 and 3)

        # When calculating the length, subtract 1 again not to include the previously called ']'
        entities.append(MessageEntityTextUrl(offset=vui[0], length=vui[1] - vui[0] - 1, url=link_url))

    # After the message is clean from links, handle all the indicator flags
    indicator_flags = {
        '*': None,
        '_': None,
        '`': None
    }

    # Iterate over the list to find the indicators of entities
    for i, c in enumerate(msg):
        # Only perform further check if the current character is an indicator
        if c in indicator_flags:
            # If it is the first time we find this indicator, update its index
            if indicator_flags[c] is None:
                indicator_flags[c] = i

            # Otherwise, it means that we found it before. Hence, the message entity *is* complete
            else:
                # Then we have found a new whole valid entity
                offset = indicator_flags[c]
                length = i - offset - 1  # Subtract -1 not to include the indicator itself

                # Add the corresponding entity
                if c == '*':
                    entities.append(MessageEntityBold(offset=offset, length=length))

                elif c == '_':
                    entities.append(MessageEntityItalic(offset=offset, length=length))

                elif c == '`':
                    entities.append(MessageEntityCode(offset=offset, length=length))

                # Clear the flag to start over with this indicator
                indicator_flags[c] = None

    # Sort the entities by their offset first
    entities = sorted(entities, key=lambda e: e.offset)

    # Now that all the entities have been found and sorted, remove
    # their indicators from the message and update the offsets
    for entity in entities:
        if type(entity) is not MessageEntityTextUrl:
            # Clean the message from the current entity's indicators
            del msg[entity.offset + entity.length + 1]
            del msg[entity.offset]

            # Iterate over all the entities but the current
            for subentity in [e for e in entities if e is not entity]:
                # First case, one in one out: so*me_th_in*g.
                # In this case, the current entity length is decreased by two,
                # and all the subentities offset decreases 1
                if (subentity.offset > entity.offset and
                        subentity.offset + subentity.length < entity.offset + entity.length):
                    entity.length -= 2
                    subentity.offset -= 1

                # Second case, both inside: so*me_th*in_g.
                # In this case, the current entity length is decreased by one,
                # and all the subentities offset and length decrease 1
                elif (entity.offset < subentity.offset < entity.offset + entity.length and
                      subentity.offset + subentity.length > entity.offset + entity.length):
                    entity.length -= 1
                    subentity.offset -= 1
                    subentity.length -= 1

                # Third case, both outside: so*me*th_in_g.
                # In this case, the current entity is left untouched,
                # and all the subentities offset decreases 2
                elif subentity.offset > entity.offset + entity.length:
                    subentity.offset -= 2

    # Finally, we can join our poor mutilated message back and return
    msg = ''.join(msg)
    return msg, entities



from telethon.tl.mtproto_request import MTProtoRequest


class ImportBotAuthorizationRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.importBotAuthorization#67a3ff2c flags:int api_id:int api_hash:string bot_auth_token:string = auth.Authorization"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x67a3ff2c

    def __init__(self, flags, api_id, api_hash, bot_auth_token):
        """
        :param flags: Telegram type: «int».
        :param api_id: Telegram type: «int».
        :param api_hash: Telegram type: «string».
        :param bot_auth_token: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.flags = flags
        self.api_id = api_id
        self.api_hash = api_hash
        self.bot_auth_token = bot_auth_token

    def on_send(self, writer):
        writer.write_int(ImportBotAuthorizationRequest.constructor_id, signed=False)
        writer.write_int(self.flags)
        writer.write_int(self.api_id)
        writer.tgwrite_string(self.api_hash)
        writer.tgwrite_string(self.bot_auth_token)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ImportBotAuthorizationRequest(None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'auth.importBotAuthorization#67a3ff2c flags:int api_id:int api_hash:string bot_auth_token:string = auth.Authorization'

    def __str__(self):
        return '(auth.importBotAuthorization (ID: 0x67a3ff2c) = (flags={}, api_id={}, api_hash={}, bot_auth_token={}))'.format(str(self.flags), str(self.api_id), str(self.api_hash), str(self.bot_auth_token))



from telethon.tl.mtproto_request import MTProtoRequest


class ImportedContacts(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.importedContacts#ad524315 imported:Vector<ImportedContact> retry_contacts:Vector<long> users:Vector<User> = contacts.ImportedContacts"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xad524315

    def __init__(self, imported, retry_contacts, users):
        """
        :param imported: Telegram type: «ImportedContact». Must be a list.
        :param retry_contacts: Telegram type: «long». Must be a list.
        :param users: Telegram type: «User». Must be a list.
        """
        super().__init__()

        self.imported = imported
        self.retry_contacts = retry_contacts
        self.users = users

    def on_send(self, writer):
        writer.write_int(ImportedContacts.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.imported))
        for imported_item in self.imported:
            imported_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.retry_contacts))
        for retry_contacts_item in self.retry_contacts:
            writer.write_long(retry_contacts_item)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ImportedContacts(None, None, None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.imported = []  # Initialize an empty list
        imported_len = reader.read_int()
        for _ in range(imported_len):
            imported_item = reader.tgread_object()
            self.imported.append(imported_item)

        reader.read_int()  # Vector's constructor ID
        self.retry_contacts = []  # Initialize an empty list
        retry_contacts_len = reader.read_int()
        for _ in range(retry_contacts_len):
            retry_contacts_item = reader.read_long()
            self.retry_contacts.append(retry_contacts_item)

        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.tgread_object()
            self.users.append(users_item)

    def __repr__(self):
        return 'contacts.importedContacts#ad524315 imported:Vector<ImportedContact> retry_contacts:Vector<long> users:Vector<User> = contacts.ImportedContacts'

    def __str__(self):
        return '(contacts.importedContacts (ID: 0xad524315) = (imported={}, retry_contacts={}, users={}))'.format(None if not self.imported else [str(_) for _ in self.imported], None if not self.retry_contacts else [str(_) for _ in self.retry_contacts], None if not self.users else [str(_) for _ in self.users])



from telethon.tl.mtproto_request import MTProtoRequest


class Authorization(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.authorization#cd050916 flags:None tmp_sessions:flags.0?int user:User = auth.Authorization"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xcd050916

    def __init__(self, user, tmp_sessions=None):
        """
        :param tmp_sessions: Telegram type: «int».
        :param user: Telegram type: «User».
        """
        super().__init__()

        self.tmp_sessions = tmp_sessions
        self.user = user

    def on_send(self, writer):
        writer.write_int(Authorization.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.tmp_sessions else 0
        writer.write_int(flags)

        if self.tmp_sessions:
            writer.write_int(self.tmp_sessions)

        self.user.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Authorization(None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.tmp_sessions = reader.read_int()

        self.user = reader.tgread_object()

    def __repr__(self):
        return 'auth.authorization#cd050916 flags:None tmp_sessions:flags.0?int user:User = auth.Authorization'

    def __str__(self):
        return '(auth.authorization (ID: 0xcd050916) = (tmp_sessions={}, user={}))'.format(str(self.tmp_sessions), str(self.user))



from telethon.tl.mtproto_request import MTProtoRequest


class ChatEmpty(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    chatEmpty#9ba2d800 id:int = Chat"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9ba2d800

    def __init__(self, id):
        """
        :param id: Telegram type: «int».
        """
        super().__init__()

        self.id = id

    def on_send(self, writer):
        writer.write_int(ChatEmpty.constructor_id, signed=False)
        writer.write_int(self.id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChatEmpty(None)

    def on_response(self, reader):
        self.id = reader.read_int()

    def __repr__(self):
        return 'chatEmpty#9ba2d800 id:int = Chat'

    def __str__(self):
        return '(chatEmpty (ID: 0x9ba2d800) = (id={}))'.format(str(self.id))



from telethon.tl.mtproto_request import MTProtoRequest


class Error(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    error#c4b9f9bb code:int text:string = Error"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xc4b9f9bb

    def __init__(self, code, text):
        """
        :param code: Telegram type: «int».
        :param text: Telegram type: «string».
        """
        super().__init__()

        self.code = code
        self.text = text

    def on_send(self, writer):
        writer.write_int(Error.constructor_id, signed=False)
        writer.write_int(self.code)
        writer.tgwrite_string(self.text)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Error(None, None)

    def on_response(self, reader):
        self.code = reader.read_int()
        self.text = reader.tgread_string()

    def __repr__(self):
        return 'error#c4b9f9bb code:int text:string = Error'

    def __str__(self):
        return '(error (ID: 0xc4b9f9bb) = (code={}, text={}))'.format(str(self.code), str(self.text))



from telethon.tl.mtproto_request import MTProtoRequest


class InputMediaGifExternal(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputMediaGifExternal#4843b0fd url:string q:string = InputMedia"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x4843b0fd

    def __init__(self, url, q):
        """
        :param url: Telegram type: «string».
        :param q: Telegram type: «string».
        """
        super().__init__()

        self.url = url
        self.q = q

    def on_send(self, writer):
        writer.write_int(InputMediaGifExternal.constructor_id, signed=False)
        writer.tgwrite_string(self.url)
        writer.tgwrite_string(self.q)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputMediaGifExternal(None, None)

    def on_response(self, reader):
        self.url = reader.tgread_string()
        self.q = reader.tgread_string()

    def __repr__(self):
        return 'inputMediaGifExternal#4843b0fd url:string q:string = InputMedia'

    def __str__(self):
        return '(inputMediaGifExternal (ID: 0x4843b0fd) = (url={}, q={}))'.format(str(self.url), str(self.q))



from telethon.tl.types.res_pq import ResPQ
from telethon.tl.types.p_q_inner_data import PQInnerData
from telethon.tl.types.server_dh_params_fail import ServerDHParamsFail
from telethon.tl.types.server_dh_params_ok import ServerDHParamsOk
from telethon.tl.types.server_dh_inner_data import ServerDHInnerData
from telethon.tl.types.client_dh_inner_data import ClientDHInnerData
from telethon.tl.types.dh_gen_ok import DhGenOk
from telethon.tl.types.dh_gen_retry import DhGenRetry
from telethon.tl.types.dh_gen_fail import DhGenFail
from telethon.tl.types.msgs_ack import MsgsAck
from telethon.tl.types.bad_msg_notification import BadMsgNotification
from telethon.tl.types.bad_server_salt import BadServerSalt
from telethon.tl.types.msgs_state_req import MsgsStateReq
from telethon.tl.types.msgs_state_info import MsgsStateInfo
from telethon.tl.types.msgs_all_info import MsgsAllInfo
from telethon.tl.types.msg_detailed_info import MsgDetailedInfo
from telethon.tl.types.msg_new_detailed_info import MsgNewDetailedInfo
from telethon.tl.types.msg_resend_req import MsgResendReq
from telethon.tl.types.rpc_error import RpcError
from telethon.tl.types.rpc_answer_unknown import RpcAnswerUnknown
from telethon.tl.types.rpc_answer_dropped_running import RpcAnswerDroppedRunning
from telethon.tl.types.rpc_answer_dropped import RpcAnswerDropped
from telethon.tl.types.future_salt import FutureSalt
from telethon.tl.types.future_salts import FutureSalts
from telethon.tl.types.pong import Pong
from telethon.tl.types.destroy_session_ok import DestroySessionOk
from telethon.tl.types.destroy_session_none import DestroySessionNone
from telethon.tl.types.new_session_created import NewSessionCreated
from telethon.tl.types.http_wait import HttpWait
from telethon.tl.types.error import Error
from telethon.tl.types.null import Null
from telethon.tl.types.input_peer_empty import InputPeerEmpty
from telethon.tl.types.input_peer_self import InputPeerSelf
from telethon.tl.types.input_peer_chat import InputPeerChat
from telethon.tl.types.input_peer_user import InputPeerUser
from telethon.tl.types.input_peer_channel import InputPeerChannel
from telethon.tl.types.input_user_empty import InputUserEmpty
from telethon.tl.types.input_user_self import InputUserSelf
from telethon.tl.types.input_user import InputUser
from telethon.tl.types.input_phone_contact import InputPhoneContact
from telethon.tl.types.input_file import InputFile
from telethon.tl.types.input_file_big import InputFileBig
from telethon.tl.types.input_media_empty import InputMediaEmpty
from telethon.tl.types.input_media_uploaded_photo import InputMediaUploadedPhoto
from telethon.tl.types.input_media_photo import InputMediaPhoto
from telethon.tl.types.input_media_geo_point import InputMediaGeoPoint
from telethon.tl.types.input_media_contact import InputMediaContact
from telethon.tl.types.input_media_uploaded_document import InputMediaUploadedDocument
from telethon.tl.types.input_media_uploaded_thumb_document import InputMediaUploadedThumbDocument
from telethon.tl.types.input_media_document import InputMediaDocument
from telethon.tl.types.input_media_venue import InputMediaVenue
from telethon.tl.types.input_media_gif_external import InputMediaGifExternal
from telethon.tl.types.input_media_photo_external import InputMediaPhotoExternal
from telethon.tl.types.input_media_document_external import InputMediaDocumentExternal
from telethon.tl.types.input_media_game import InputMediaGame
from telethon.tl.types.input_chat_photo_empty import InputChatPhotoEmpty
from telethon.tl.types.input_chat_uploaded_photo import InputChatUploadedPhoto
from telethon.tl.types.input_chat_photo import InputChatPhoto
from telethon.tl.types.input_geo_point_empty import InputGeoPointEmpty
from telethon.tl.types.input_geo_point import InputGeoPoint
from telethon.tl.types.input_photo_empty import InputPhotoEmpty
from telethon.tl.types.input_photo import InputPhoto
from telethon.tl.types.input_file_location import InputFileLocation
from telethon.tl.types.input_encrypted_file_location import InputEncryptedFileLocation
from telethon.tl.types.input_document_file_location import InputDocumentFileLocation
from telethon.tl.types.input_app_event import InputAppEvent
from telethon.tl.types.peer_user import PeerUser
from telethon.tl.types.peer_chat import PeerChat
from telethon.tl.types.peer_channel import PeerChannel
from telethon.tl.types.file_location_unavailable import FileLocationUnavailable
from telethon.tl.types.file_location import FileLocation
from telethon.tl.types.user_empty import UserEmpty
from telethon.tl.types.user import User
from telethon.tl.types.user_profile_photo_empty import UserProfilePhotoEmpty
from telethon.tl.types.user_profile_photo import UserProfilePhoto
from telethon.tl.types.user_status_empty import UserStatusEmpty
from telethon.tl.types.user_status_online import UserStatusOnline
from telethon.tl.types.user_status_offline import UserStatusOffline
from telethon.tl.types.user_status_recently import UserStatusRecently
from telethon.tl.types.user_status_last_week import UserStatusLastWeek
from telethon.tl.types.user_status_last_month import UserStatusLastMonth
from telethon.tl.types.chat_empty import ChatEmpty
from telethon.tl.types.chat import Chat
from telethon.tl.types.chat_forbidden import ChatForbidden
from telethon.tl.types.channel import Channel
from telethon.tl.types.channel_forbidden import ChannelForbidden
from telethon.tl.types.chat_full import ChatFull
from telethon.tl.types.channel_full import ChannelFull
from telethon.tl.types.chat_participant import ChatParticipant
from telethon.tl.types.chat_participant_creator import ChatParticipantCreator
from telethon.tl.types.chat_participant_admin import ChatParticipantAdmin
from telethon.tl.types.chat_participants_forbidden import ChatParticipantsForbidden
from telethon.tl.types.chat_participants import ChatParticipants
from telethon.tl.types.chat_photo_empty import ChatPhotoEmpty
from telethon.tl.types.chat_photo import ChatPhoto
from telethon.tl.types.message_empty import MessageEmpty
from telethon.tl.types.message import Message
from telethon.tl.types.message_service import MessageService
from telethon.tl.types.message_media_empty import MessageMediaEmpty
from telethon.tl.types.message_media_photo import MessageMediaPhoto
from telethon.tl.types.message_media_geo import MessageMediaGeo
from telethon.tl.types.message_media_contact import MessageMediaContact
from telethon.tl.types.message_media_unsupported import MessageMediaUnsupported
from telethon.tl.types.message_media_document import MessageMediaDocument
from telethon.tl.types.message_media_web_page import MessageMediaWebPage
from telethon.tl.types.message_media_venue import MessageMediaVenue
from telethon.tl.types.message_media_game import MessageMediaGame
from telethon.tl.types.message_action_empty import MessageActionEmpty
from telethon.tl.types.message_action_chat_create import MessageActionChatCreate
from telethon.tl.types.message_action_chat_edit_title import MessageActionChatEditTitle
from telethon.tl.types.message_action_chat_edit_photo import MessageActionChatEditPhoto
from telethon.tl.types.message_action_chat_delete_photo import MessageActionChatDeletePhoto
from telethon.tl.types.message_action_chat_add_user import MessageActionChatAddUser
from telethon.tl.types.message_action_chat_delete_user import MessageActionChatDeleteUser
from telethon.tl.types.message_action_chat_joined_by_link import MessageActionChatJoinedByLink
from telethon.tl.types.message_action_channel_create import MessageActionChannelCreate
from telethon.tl.types.message_action_chat_migrate_to import MessageActionChatMigrateTo
from telethon.tl.types.message_action_channel_migrate_from import MessageActionChannelMigrateFrom
from telethon.tl.types.message_action_pin_message import MessageActionPinMessage
from telethon.tl.types.message_action_history_clear import MessageActionHistoryClear
from telethon.tl.types.message_action_game_score import MessageActionGameScore
from telethon.tl.types.dialog import Dialog
from telethon.tl.types.photo_empty import PhotoEmpty
from telethon.tl.types.photo import Photo
from telethon.tl.types.photo_size_empty import PhotoSizeEmpty
from telethon.tl.types.photo_size import PhotoSize
from telethon.tl.types.photo_cached_size import PhotoCachedSize
from telethon.tl.types.geo_point_empty import GeoPointEmpty
from telethon.tl.types.geo_point import GeoPoint
from telethon.tl.types.input_notify_peer import InputNotifyPeer
from telethon.tl.types.input_notify_users import InputNotifyUsers
from telethon.tl.types.input_notify_chats import InputNotifyChats
from telethon.tl.types.input_notify_all import InputNotifyAll
from telethon.tl.types.input_peer_notify_events_empty import InputPeerNotifyEventsEmpty
from telethon.tl.types.input_peer_notify_events_all import InputPeerNotifyEventsAll
from telethon.tl.types.input_peer_notify_settings import InputPeerNotifySettings
from telethon.tl.types.peer_notify_events_empty import PeerNotifyEventsEmpty
from telethon.tl.types.peer_notify_events_all import PeerNotifyEventsAll
from telethon.tl.types.peer_notify_settings_empty import PeerNotifySettingsEmpty
from telethon.tl.types.peer_notify_settings import PeerNotifySettings
from telethon.tl.types.peer_settings import PeerSettings
from telethon.tl.types.wall_paper import WallPaper
from telethon.tl.types.wall_paper_solid import WallPaperSolid
from telethon.tl.types.input_report_reason_spam import InputReportReasonSpam
from telethon.tl.types.input_report_reason_violence import InputReportReasonViolence
from telethon.tl.types.input_report_reason_pornography import InputReportReasonPornography
from telethon.tl.types.input_report_reason_other import InputReportReasonOther
from telethon.tl.types.user_full import UserFull
from telethon.tl.types.contact import Contact
from telethon.tl.types.imported_contact import ImportedContact
from telethon.tl.types.contact_blocked import ContactBlocked
from telethon.tl.types.contact_status import ContactStatus
from telethon.tl.types.input_messages_filter_empty import InputMessagesFilterEmpty
from telethon.tl.types.input_messages_filter_photos import InputMessagesFilterPhotos
from telethon.tl.types.input_messages_filter_video import InputMessagesFilterVideo
from telethon.tl.types.input_messages_filter_photo_video import InputMessagesFilterPhotoVideo
from telethon.tl.types.input_messages_filter_photo_video_documents import InputMessagesFilterPhotoVideoDocuments
from telethon.tl.types.input_messages_filter_document import InputMessagesFilterDocument
from telethon.tl.types.input_messages_filter_url import InputMessagesFilterUrl
from telethon.tl.types.input_messages_filter_gif import InputMessagesFilterGif
from telethon.tl.types.input_messages_filter_voice import InputMessagesFilterVoice
from telethon.tl.types.input_messages_filter_music import InputMessagesFilterMusic
from telethon.tl.types.input_messages_filter_chat_photos import InputMessagesFilterChatPhotos
from telethon.tl.types.update_new_message import UpdateNewMessage
from telethon.tl.types.update_message_id import UpdateMessageID
from telethon.tl.types.update_delete_messages import UpdateDeleteMessages
from telethon.tl.types.update_user_typing import UpdateUserTyping
from telethon.tl.types.update_chat_user_typing import UpdateChatUserTyping
from telethon.tl.types.update_chat_participants import UpdateChatParticipants
from telethon.tl.types.update_user_status import UpdateUserStatus
from telethon.tl.types.update_user_name import UpdateUserName
from telethon.tl.types.update_user_photo import UpdateUserPhoto
from telethon.tl.types.update_contact_registered import UpdateContactRegistered
from telethon.tl.types.update_contact_link import UpdateContactLink
from telethon.tl.types.update_new_authorization import UpdateNewAuthorization
from telethon.tl.types.update_new_encrypted_message import UpdateNewEncryptedMessage
from telethon.tl.types.update_encrypted_chat_typing import UpdateEncryptedChatTyping
from telethon.tl.types.update_encryption import UpdateEncryption
from telethon.tl.types.update_encrypted_messages_read import UpdateEncryptedMessagesRead
from telethon.tl.types.update_chat_participant_add import UpdateChatParticipantAdd
from telethon.tl.types.update_chat_participant_delete import UpdateChatParticipantDelete
from telethon.tl.types.update_dc_options import UpdateDcOptions
from telethon.tl.types.update_user_blocked import UpdateUserBlocked
from telethon.tl.types.update_notify_settings import UpdateNotifySettings
from telethon.tl.types.update_service_notification import UpdateServiceNotification
from telethon.tl.types.update_privacy import UpdatePrivacy
from telethon.tl.types.update_user_phone import UpdateUserPhone
from telethon.tl.types.update_read_history_inbox import UpdateReadHistoryInbox
from telethon.tl.types.update_read_history_outbox import UpdateReadHistoryOutbox
from telethon.tl.types.update_web_page import UpdateWebPage
from telethon.tl.types.update_read_messages_contents import UpdateReadMessagesContents
from telethon.tl.types.update_channel_too_long import UpdateChannelTooLong
from telethon.tl.types.update_channel import UpdateChannel
from telethon.tl.types.update_new_channel_message import UpdateNewChannelMessage
from telethon.tl.types.update_read_channel_inbox import UpdateReadChannelInbox
from telethon.tl.types.update_delete_channel_messages import UpdateDeleteChannelMessages
from telethon.tl.types.update_channel_message_views import UpdateChannelMessageViews
from telethon.tl.types.update_chat_admins import UpdateChatAdmins
from telethon.tl.types.update_chat_participant_admin import UpdateChatParticipantAdmin
from telethon.tl.types.update_new_sticker_set import UpdateNewStickerSet
from telethon.tl.types.update_sticker_sets_order import UpdateStickerSetsOrder
from telethon.tl.types.update_sticker_sets import UpdateStickerSets
from telethon.tl.types.update_saved_gifs import UpdateSavedGifs
from telethon.tl.types.update_bot_inline_query import UpdateBotInlineQuery
from telethon.tl.types.update_bot_inline_send import UpdateBotInlineSend
from telethon.tl.types.update_edit_channel_message import UpdateEditChannelMessage
from telethon.tl.types.update_channel_pinned_message import UpdateChannelPinnedMessage
from telethon.tl.types.update_bot_callback_query import UpdateBotCallbackQuery
from telethon.tl.types.update_edit_message import UpdateEditMessage
from telethon.tl.types.update_inline_bot_callback_query import UpdateInlineBotCallbackQuery
from telethon.tl.types.update_read_channel_outbox import UpdateReadChannelOutbox
from telethon.tl.types.update_draft_message import UpdateDraftMessage
from telethon.tl.types.update_read_featured_stickers import UpdateReadFeaturedStickers
from telethon.tl.types.update_recent_stickers import UpdateRecentStickers
from telethon.tl.types.update_config import UpdateConfig
from telethon.tl.types.update_pts_changed import UpdatePtsChanged
from telethon.tl.types.updates_too_long import UpdatesTooLong
from telethon.tl.types.update_short_message import UpdateShortMessage
from telethon.tl.types.update_short_chat_message import UpdateShortChatMessage
from telethon.tl.types.update_short import UpdateShort
from telethon.tl.types.updates_combined import UpdatesCombined
from telethon.tl.types.updates_tg import UpdatesTg
from telethon.tl.types.update_short_sent_message import UpdateShortSentMessage
from telethon.tl.types.dc_option import DcOption
from telethon.tl.types.config import Config
from telethon.tl.types.nearest_dc import NearestDc
from telethon.tl.types.encrypted_chat_empty import EncryptedChatEmpty
from telethon.tl.types.encrypted_chat_waiting import EncryptedChatWaiting
from telethon.tl.types.encrypted_chat_requested import EncryptedChatRequested
from telethon.tl.types.encrypted_chat import EncryptedChat
from telethon.tl.types.encrypted_chat_discarded import EncryptedChatDiscarded
from telethon.tl.types.input_encrypted_chat import InputEncryptedChat
from telethon.tl.types.encrypted_file_empty import EncryptedFileEmpty
from telethon.tl.types.encrypted_file import EncryptedFile
from telethon.tl.types.input_encrypted_file_empty import InputEncryptedFileEmpty
from telethon.tl.types.input_encrypted_file_uploaded import InputEncryptedFileUploaded
from telethon.tl.types.input_encrypted_file import InputEncryptedFile
from telethon.tl.types.input_encrypted_file_big_uploaded import InputEncryptedFileBigUploaded
from telethon.tl.types.encrypted_message import EncryptedMessage
from telethon.tl.types.encrypted_message_service import EncryptedMessageService
from telethon.tl.types.input_document_empty import InputDocumentEmpty
from telethon.tl.types.input_document import InputDocument
from telethon.tl.types.document_empty import DocumentEmpty
from telethon.tl.types.document import Document
from telethon.tl.types.notify_peer import NotifyPeer
from telethon.tl.types.notify_users import NotifyUsers
from telethon.tl.types.notify_chats import NotifyChats
from telethon.tl.types.notify_all import NotifyAll
from telethon.tl.types.send_message_typing_action import SendMessageTypingAction
from telethon.tl.types.send_message_cancel_action import SendMessageCancelAction
from telethon.tl.types.send_message_record_video_action import SendMessageRecordVideoAction
from telethon.tl.types.send_message_upload_video_action import SendMessageUploadVideoAction
from telethon.tl.types.send_message_record_audio_action import SendMessageRecordAudioAction
from telethon.tl.types.send_message_upload_audio_action import SendMessageUploadAudioAction
from telethon.tl.types.send_message_upload_photo_action import SendMessageUploadPhotoAction
from telethon.tl.types.send_message_upload_document_action import SendMessageUploadDocumentAction
from telethon.tl.types.send_message_geo_location_action import SendMessageGeoLocationAction
from telethon.tl.types.send_message_choose_contact_action import SendMessageChooseContactAction
from telethon.tl.types.send_message_game_play_action import SendMessageGamePlayAction
from telethon.tl.types.send_message_game_stop_action import SendMessageGameStopAction
from telethon.tl.types.input_privacy_key_status_timestamp import InputPrivacyKeyStatusTimestamp
from telethon.tl.types.input_privacy_key_chat_invite import InputPrivacyKeyChatInvite
from telethon.tl.types.privacy_key_status_timestamp import PrivacyKeyStatusTimestamp
from telethon.tl.types.privacy_key_chat_invite import PrivacyKeyChatInvite
from telethon.tl.types.input_privacy_value_allow_contacts import InputPrivacyValueAllowContacts
from telethon.tl.types.input_privacy_value_allow_all import InputPrivacyValueAllowAll
from telethon.tl.types.input_privacy_value_allow_users import InputPrivacyValueAllowUsers
from telethon.tl.types.input_privacy_value_disallow_contacts import InputPrivacyValueDisallowContacts
from telethon.tl.types.input_privacy_value_disallow_all import InputPrivacyValueDisallowAll
from telethon.tl.types.input_privacy_value_disallow_users import InputPrivacyValueDisallowUsers
from telethon.tl.types.privacy_value_allow_contacts import PrivacyValueAllowContacts
from telethon.tl.types.privacy_value_allow_all import PrivacyValueAllowAll
from telethon.tl.types.privacy_value_allow_users import PrivacyValueAllowUsers
from telethon.tl.types.privacy_value_disallow_contacts import PrivacyValueDisallowContacts
from telethon.tl.types.privacy_value_disallow_all import PrivacyValueDisallowAll
from telethon.tl.types.privacy_value_disallow_users import PrivacyValueDisallowUsers
from telethon.tl.types.account_days_ttl import AccountDaysTTL
from telethon.tl.types.document_attribute_image_size import DocumentAttributeImageSize
from telethon.tl.types.document_attribute_animated import DocumentAttributeAnimated
from telethon.tl.types.document_attribute_sticker import DocumentAttributeSticker
from telethon.tl.types.document_attribute_video import DocumentAttributeVideo
from telethon.tl.types.document_attribute_audio import DocumentAttributeAudio
from telethon.tl.types.document_attribute_filename import DocumentAttributeFilename
from telethon.tl.types.document_attribute_has_stickers import DocumentAttributeHasStickers
from telethon.tl.types.sticker_pack import StickerPack
from telethon.tl.types.disabled_feature import DisabledFeature
from telethon.tl.types.contact_link_unknown import ContactLinkUnknown
from telethon.tl.types.contact_link_none import ContactLinkNone
from telethon.tl.types.contact_link_has_phone import ContactLinkHasPhone
from telethon.tl.types.contact_link_contact import ContactLinkContact
from telethon.tl.types.web_page_empty import WebPageEmpty
from telethon.tl.types.web_page_pending import WebPagePending
from telethon.tl.types.web_page import WebPage
from telethon.tl.types.authorization import Authorization
from telethon.tl.types.received_notify_message import ReceivedNotifyMessage
from telethon.tl.types.chat_invite_empty import ChatInviteEmpty
from telethon.tl.types.chat_invite_exported import ChatInviteExported
from telethon.tl.types.chat_invite_already import ChatInviteAlready
from telethon.tl.types.chat_invite import ChatInvite
from telethon.tl.types.input_sticker_set_empty import InputStickerSetEmpty
from telethon.tl.types.input_sticker_set_id import InputStickerSetID
from telethon.tl.types.input_sticker_set_short_name import InputStickerSetShortName
from telethon.tl.types.sticker_set import StickerSet
from telethon.tl.types.bot_command import BotCommand
from telethon.tl.types.bot_info import BotInfo
from telethon.tl.types.keyboard_button import KeyboardButton
from telethon.tl.types.keyboard_button_url import KeyboardButtonUrl
from telethon.tl.types.keyboard_button_callback import KeyboardButtonCallback
from telethon.tl.types.keyboard_button_request_phone import KeyboardButtonRequestPhone
from telethon.tl.types.keyboard_button_request_geo_location import KeyboardButtonRequestGeoLocation
from telethon.tl.types.keyboard_button_switch_inline import KeyboardButtonSwitchInline
from telethon.tl.types.keyboard_button_game import KeyboardButtonGame
from telethon.tl.types.keyboard_button_row import KeyboardButtonRow
from telethon.tl.types.reply_keyboard_hide import ReplyKeyboardHide
from telethon.tl.types.reply_keyboard_force_reply import ReplyKeyboardForceReply
from telethon.tl.types.reply_keyboard_markup import ReplyKeyboardMarkup
from telethon.tl.types.reply_inline_markup import ReplyInlineMarkup
from telethon.tl.types.message_entity_unknown import MessageEntityUnknown
from telethon.tl.types.message_entity_mention import MessageEntityMention
from telethon.tl.types.message_entity_hashtag import MessageEntityHashtag
from telethon.tl.types.message_entity_bot_command import MessageEntityBotCommand
from telethon.tl.types.message_entity_url import MessageEntityUrl
from telethon.tl.types.message_entity_email import MessageEntityEmail
from telethon.tl.types.message_entity_bold import MessageEntityBold
from telethon.tl.types.message_entity_italic import MessageEntityItalic
from telethon.tl.types.message_entity_code import MessageEntityCode
from telethon.tl.types.message_entity_pre import MessageEntityPre
from telethon.tl.types.message_entity_text_url import MessageEntityTextUrl
from telethon.tl.types.message_entity_mention_name import MessageEntityMentionName
from telethon.tl.types.input_message_entity_mention_name import InputMessageEntityMentionName
from telethon.tl.types.input_channel_empty import InputChannelEmpty
from telethon.tl.types.input_channel import InputChannel
from telethon.tl.types.message_range import MessageRange
from telethon.tl.types.channel_messages_filter_empty import ChannelMessagesFilterEmpty
from telethon.tl.types.channel_messages_filter import ChannelMessagesFilter
from telethon.tl.types.channel_participant import ChannelParticipant
from telethon.tl.types.channel_participant_self import ChannelParticipantSelf
from telethon.tl.types.channel_participant_moderator import ChannelParticipantModerator
from telethon.tl.types.channel_participant_editor import ChannelParticipantEditor
from telethon.tl.types.channel_participant_kicked import ChannelParticipantKicked
from telethon.tl.types.channel_participant_creator import ChannelParticipantCreator
from telethon.tl.types.channel_participants_recent import ChannelParticipantsRecent
from telethon.tl.types.channel_participants_admins import ChannelParticipantsAdmins
from telethon.tl.types.channel_participants_kicked import ChannelParticipantsKicked
from telethon.tl.types.channel_participants_bots import ChannelParticipantsBots
from telethon.tl.types.channel_role_empty import ChannelRoleEmpty
from telethon.tl.types.channel_role_moderator import ChannelRoleModerator
from telethon.tl.types.channel_role_editor import ChannelRoleEditor
from telethon.tl.types.found_gif import FoundGif
from telethon.tl.types.found_gif_cached import FoundGifCached
from telethon.tl.types.input_bot_inline_message_media_auto import InputBotInlineMessageMediaAuto
from telethon.tl.types.input_bot_inline_message_text import InputBotInlineMessageText
from telethon.tl.types.input_bot_inline_message_media_geo import InputBotInlineMessageMediaGeo
from telethon.tl.types.input_bot_inline_message_media_venue import InputBotInlineMessageMediaVenue
from telethon.tl.types.input_bot_inline_message_media_contact import InputBotInlineMessageMediaContact
from telethon.tl.types.input_bot_inline_message_game import InputBotInlineMessageGame
from telethon.tl.types.input_bot_inline_result import InputBotInlineResult
from telethon.tl.types.input_bot_inline_result_photo import InputBotInlineResultPhoto
from telethon.tl.types.input_bot_inline_result_document import InputBotInlineResultDocument
from telethon.tl.types.input_bot_inline_result_game import InputBotInlineResultGame
from telethon.tl.types.bot_inline_message_media_auto import BotInlineMessageMediaAuto
from telethon.tl.types.bot_inline_message_text import BotInlineMessageText
from telethon.tl.types.bot_inline_message_media_geo import BotInlineMessageMediaGeo
from telethon.tl.types.bot_inline_message_media_venue import BotInlineMessageMediaVenue
from telethon.tl.types.bot_inline_message_media_contact import BotInlineMessageMediaContact
from telethon.tl.types.bot_inline_result import BotInlineResult
from telethon.tl.types.bot_inline_media_result import BotInlineMediaResult
from telethon.tl.types.exported_message_link import ExportedMessageLink
from telethon.tl.types.message_fwd_header import MessageFwdHeader
from telethon.tl.types.input_bot_inline_message_id import InputBotInlineMessageID
from telethon.tl.types.inline_bot_switch_pm import InlineBotSwitchPM
from telethon.tl.types.top_peer import TopPeer
from telethon.tl.types.top_peer_category_bots_pm import TopPeerCategoryBotsPM
from telethon.tl.types.top_peer_category_bots_inline import TopPeerCategoryBotsInline
from telethon.tl.types.top_peer_category_correspondents import TopPeerCategoryCorrespondents
from telethon.tl.types.top_peer_category_groups import TopPeerCategoryGroups
from telethon.tl.types.top_peer_category_channels import TopPeerCategoryChannels
from telethon.tl.types.top_peer_category_peers import TopPeerCategoryPeers
from telethon.tl.types.draft_message_empty import DraftMessageEmpty
from telethon.tl.types.draft_message import DraftMessage
from telethon.tl.types.sticker_set_covered import StickerSetCovered
from telethon.tl.types.sticker_set_multi_covered import StickerSetMultiCovered
from telethon.tl.types.mask_coords import MaskCoords
from telethon.tl.types.input_stickered_media_photo import InputStickeredMediaPhoto
from telethon.tl.types.input_stickered_media_document import InputStickeredMediaDocument
from telethon.tl.types.game import Game
from telethon.tl.types.input_game_id import InputGameID
from telethon.tl.types.input_game_short_name import InputGameShortName
from telethon.tl.types.high_score import HighScore



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateStatusRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.updateStatus#6628562c offline:Bool = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6628562c

    def __init__(self, offline):
        """
        :param offline: Telegram type: «Bool».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.offline = offline

    def on_send(self, writer):
        writer.write_int(UpdateStatusRequest.constructor_id, signed=False)
        writer.tgwrite_bool(self.offline)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateStatusRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'account.updateStatus#6628562c offline:Bool = Bool'

    def __str__(self):
        return '(account.updateStatus (ID: 0x6628562c) = (offline={}))'.format(str(self.offline))



from telethon.tl.mtproto_request import MTProtoRequest


class MessageActionGameScore(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageActionGameScore#92a72876 game_id:long score:int = MessageAction"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x92a72876

    def __init__(self, game_id, score):
        """
        :param game_id: Telegram type: «long».
        :param score: Telegram type: «int».
        """
        super().__init__()

        self.game_id = game_id
        self.score = score

    def on_send(self, writer):
        writer.write_int(MessageActionGameScore.constructor_id, signed=False)
        writer.write_long(self.game_id)
        writer.write_int(self.score)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageActionGameScore(None, None)

    def on_response(self, reader):
        self.game_id = reader.read_long()
        self.score = reader.read_int()

    def __repr__(self):
        return 'messageActionGameScore#92a72876 game_id:long score:int = MessageAction'

    def __str__(self):
        return '(messageActionGameScore (ID: 0x92a72876) = (game_id={}, score={}))'.format(str(self.game_id), str(self.score))



from telethon.tl.mtproto_request import MTProtoRequest


class InputStickerSetShortName(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputStickerSetShortName#861cc8a0 short_name:string = InputStickerSet"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x861cc8a0

    def __init__(self, short_name):
        """
        :param short_name: Telegram type: «string».
        """
        super().__init__()

        self.short_name = short_name

    def on_send(self, writer):
        writer.write_int(InputStickerSetShortName.constructor_id, signed=False)
        writer.tgwrite_string(self.short_name)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputStickerSetShortName(None)

    def on_response(self, reader):
        self.short_name = reader.tgread_string()

    def __repr__(self):
        return 'inputStickerSetShortName#861cc8a0 short_name:string = InputStickerSet'

    def __str__(self):
        return '(inputStickerSetShortName (ID: 0x861cc8a0) = (short_name={}))'.format(str(self.short_name))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateBotCallbackQuery(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateBotCallbackQuery#e73547e1 flags:None query_id:long user_id:int peer:Peer msg_id:int chat_instance:long data:flags.0?bytes game_short_name:flags.1?string = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xe73547e1

    def __init__(self, query_id, user_id, peer, msg_id, chat_instance, data=None, game_short_name=None):
        """
        :param query_id: Telegram type: «long».
        :param user_id: Telegram type: «int».
        :param peer: Telegram type: «Peer».
        :param msg_id: Telegram type: «int».
        :param chat_instance: Telegram type: «long».
        :param data: Telegram type: «bytes».
        :param game_short_name: Telegram type: «string».
        """
        super().__init__()

        self.query_id = query_id
        self.user_id = user_id
        self.peer = peer
        self.msg_id = msg_id
        self.chat_instance = chat_instance
        self.data = data
        self.game_short_name = game_short_name

    def on_send(self, writer):
        writer.write_int(UpdateBotCallbackQuery.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.data else 0
        flags |= (1 << 1) if self.game_short_name else 0
        writer.write_int(flags)

        writer.write_long(self.query_id)
        writer.write_int(self.user_id)
        self.peer.on_send(writer)
        writer.write_int(self.msg_id)
        writer.write_long(self.chat_instance)
        if self.data:
            writer.tgwrite_bytes(self.data)

        if self.game_short_name:
            writer.tgwrite_string(self.game_short_name)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateBotCallbackQuery(None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.query_id = reader.read_long()
        self.user_id = reader.read_int()
        self.peer = reader.tgread_object()
        self.msg_id = reader.read_int()
        self.chat_instance = reader.read_long()
        if (flags & (1 << 0)) != 0:
            self.data = reader.tgread_bytes()

        if (flags & (1 << 1)) != 0:
            self.game_short_name = reader.tgread_string()

    def __repr__(self):
        return 'updateBotCallbackQuery#e73547e1 flags:None query_id:long user_id:int peer:Peer msg_id:int chat_instance:long data:flags.0?bytes game_short_name:flags.1?string = Update'

    def __str__(self):
        return '(updateBotCallbackQuery (ID: 0xe73547e1) = (query_id={}, user_id={}, peer={}, msg_id={}, chat_instance={}, data={}, game_short_name={}))'.format(str(self.query_id), str(self.user_id), str(self.peer), str(self.msg_id), str(self.chat_instance), str(self.data), str(self.game_short_name))



from telethon.tl.mtproto_request import MTProtoRequest


class ReplyInlineMarkup(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    replyInlineMarkup#48a30254 rows:Vector<KeyboardButtonRow> = ReplyMarkup"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x48a30254

    def __init__(self, rows):
        """
        :param rows: Telegram type: «KeyboardButtonRow». Must be a list.
        """
        super().__init__()

        self.rows = rows

    def on_send(self, writer):
        writer.write_int(ReplyInlineMarkup.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.rows))
        for rows_item in self.rows:
            rows_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ReplyInlineMarkup(None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.rows = []  # Initialize an empty list
        rows_len = reader.read_int()
        for _ in range(rows_len):
            rows_item = reader.tgread_object()
            self.rows.append(rows_item)

    def __repr__(self):
        return 'replyInlineMarkup#48a30254 rows:Vector<KeyboardButtonRow> = ReplyMarkup'

    def __str__(self):
        return '(replyInlineMarkup (ID: 0x48a30254) = (rows={}))'.format(None if not self.rows else [str(_) for _ in self.rows])



from telethon.tl.mtproto_request import MTProtoRequest


class DisabledFeature(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    disabledFeature#ae636f24 feature:string description:string = DisabledFeature"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xae636f24

    def __init__(self, feature, description):
        """
        :param feature: Telegram type: «string».
        :param description: Telegram type: «string».
        """
        super().__init__()

        self.feature = feature
        self.description = description

    def on_send(self, writer):
        writer.write_int(DisabledFeature.constructor_id, signed=False)
        writer.tgwrite_string(self.feature)
        writer.tgwrite_string(self.description)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DisabledFeature(None, None)

    def on_response(self, reader):
        self.feature = reader.tgread_string()
        self.description = reader.tgread_string()

    def __repr__(self):
        return 'disabledFeature#ae636f24 feature:string description:string = DisabledFeature'

    def __str__(self):
        return '(disabledFeature (ID: 0xae636f24) = (feature={}, description={}))'.format(str(self.feature), str(self.description))



import gzip
from telethon.errors import *
from time import sleep
from datetime import timedelta
from threading import Thread, RLock

import telethon.helpers as utils
from telethon.crypto import AES
from telethon.utils import BinaryWriter, BinaryReader
from telethon.tl.types import MsgsAck
from telethon.tl.all_tlobjects import tlobjects


class MtProtoSender:
    """MTProto Mobile Protocol sender (https://core.telegram.org/mtproto/description)"""
    def __init__(self, transport, session):
        self.transport = transport
        self.session = session

        self.need_confirmation = []  # Message IDs that need confirmation
        self.on_update_handlers = []

        # Store an RLock instance to make this class safely multi-threaded
        self.lock = RLock()

        # Flag used to determine whether we've received a sent request yet or not
        # We need this to avoid using the updates thread if we're waiting to read
        self.waiting_receive = False

        self.updates_thread = Thread(target=self.updates_thread_method, name='Updates thread')
        self.updates_thread_running = False
        self.updates_thread_receiving = False

        # Determine whether the received acknowledge request confirm
        # our requests or not. This is not desired until we initialize
        # our connection, because it breaks things when we call InvokeWithLayer
        # TODO There might be a better way to handle msgs_ack requests
        self.ack_requests_confirm = False

    def disconnect(self):
        """Disconnects and **stops all the running threads** if any"""
        self.set_listen_for_updates(enabled=False)
        self.transport.close()

    def add_update_handler(self, handler):
        """Adds an update handler (a method with one argument, the received
           TLObject) that is fired when there are updates available"""

        first_handler = not self.on_update_handlers
        self.on_update_handlers.append(handler)

        # If this is the first added handler,
        # we must start the thread to receive updates
        if first_handler:
            self.set_listen_for_updates(enabled=True)

    def remove_update_handler(self, handler):
        self.on_update_handlers.remove(handler)

        # If there are no more update handlers, stop the thread
        if not self.on_update_handlers:
            self.set_listen_for_updates(False)

    def generate_sequence(self, confirmed):
        """Generates the next sequence number, based on whether it
           was confirmed yet or not"""
        if confirmed:
            result = self.session.sequence * 2 + 1
            self.session.sequence += 1
            return result
        else:
            return self.session.sequence * 2

    # region Send and receive

    def send(self, request):
        """Sends the specified MTProtoRequest, previously sending any message
           which needed confirmation. This also pauses the updates thread"""

        # Only cancel the receive *if* it was the
        # updates thread who was receiving. We do
        # not want to cancel other pending requests!
        if self.updates_thread_receiving:
            self.transport.cancel_receive()

        # Now only us can be using this method
        with self.lock:
            # Set the flag to true so the updates thread stops trying to receive
            self.waiting_receive = True

            # If any message needs confirmation send an AckRequest first
            if self.need_confirmation:
                msgs_ack = MsgsAck(self.need_confirmation)
                with BinaryWriter() as writer:
                    msgs_ack.on_send(writer)
                    self.send_packet(writer.get_bytes(), msgs_ack)

                del self.need_confirmation[:]

            # Finally send our packed request
            with BinaryWriter() as writer:
                request.on_send(writer)
                self.send_packet(writer.get_bytes(), request)

            # And update the saved session
            self.session.save()

    def receive(self, request, timeout=timedelta(seconds=5)):
        """Receives the specified MTProtoRequest ("fills in it"
           the received data). This also restores the updates thread.
           An optional timeout can be specified to cancel the operation
           if no data has been read after its time delta"""

        with self.lock:
            # Don't stop trying to receive until we get the request we wanted
            while not request.confirm_received:
                seq, body = self.transport.receive(timeout)
                message, remote_msg_id, remote_sequence = self.decode_msg(body)

                with BinaryReader(message) as reader:
                    self.process_msg(remote_msg_id, remote_sequence, reader, request)

            # We can now set the flag to False thus resuming the updates thread
            self.waiting_receive = False

    # endregion

    # region Low level processing

    def send_packet(self, packet, request):
        """Sends the given packet bytes with the additional
           information of the original request. This does NOT lock the threads!"""
        request.msg_id = self.session.get_new_msg_id()

        # First calculate plain_text to encrypt it
        with BinaryWriter() as plain_writer:
            plain_writer.write_long(self.session.salt, signed=False)
            plain_writer.write_long(self.session.id, signed=False)
            plain_writer.write_long(request.msg_id)
            plain_writer.write_int(self.generate_sequence(request.confirmed))
            plain_writer.write_int(len(packet))
            plain_writer.write(packet)

            msg_key = utils.calc_msg_key(plain_writer.get_bytes())

            key, iv = utils.calc_key(self.session.auth_key.key, msg_key, True)
            cipher_text = AES.encrypt_ige(plain_writer.get_bytes(), key, iv)

        # And then finally send the encrypted packet
        with BinaryWriter() as cipher_writer:
            cipher_writer.write_long(self.session.auth_key.key_id, signed=False)
            cipher_writer.write(msg_key)
            cipher_writer.write(cipher_text)
            self.transport.send(cipher_writer.get_bytes())

    def decode_msg(self, body):
        """Decodes an received encrypted message body bytes"""
        message = None
        remote_msg_id = None
        remote_sequence = None

        with BinaryReader(body) as reader:
            if len(body) < 8:
                raise BufferError("Can't decode packet ({})".format(body))

            # TODO Check for both auth key ID and msg_key correctness
            remote_auth_key_id = reader.read_long()
            msg_key = reader.read(16)

            key, iv = utils.calc_key(self.session.auth_key.key, msg_key, False)
            plain_text = AES.decrypt_ige(reader.read(len(body) - reader.tell_position()), key, iv)

            with BinaryReader(plain_text) as plain_text_reader:
                remote_salt = plain_text_reader.read_long()
                remote_session_id = plain_text_reader.read_long()
                remote_msg_id = plain_text_reader.read_long()
                remote_sequence = plain_text_reader.read_int()
                msg_len = plain_text_reader.read_int()
                message = plain_text_reader.read(msg_len)

        return message, remote_msg_id, remote_sequence

    def process_msg(self, msg_id, sequence, reader, request=None):
        """Processes and handles a Telegram message"""

        # TODO Check salt, session_id and sequence_number
        self.need_confirmation.append(msg_id)

        code = reader.read_int(signed=False)
        reader.seek(-4)

        # The following codes are "parsed manually"
        if code == 0xf35c6d01:  # rpc_result, (response of an RPC call, i.e., we sent a request)
            return self.handle_rpc_result(msg_id, sequence, reader, request)

        if code == 0x73f1f8dc:  # msg_container
            return self.handle_container(msg_id, sequence, reader, request)
        if code == 0x3072cfa1:  # gzip_packed
            return self.handle_gzip_packed(msg_id, sequence, reader, request)
        if code == 0xedab447b:  # bad_server_salt
            return self.handle_bad_server_salt(msg_id, sequence, reader, request)
        if code == 0xa7eff811:  # bad_msg_notification
            return self.handle_bad_msg_notification(msg_id, sequence, reader)

        # msgs_ack, it may handle the request we wanted
        if self.ack_requests_confirm and code == 0x62d6b459:
            ack = reader.tgread_object()
            if request and request.msg_id in ack.msg_ids:
                request.confirm_received = True
            return False

        # If the code is not parsed manually, then it was parsed by the code generator!
        # In this case, we will simply treat the incoming TLObject as an Update,
        # if we can first find a matching TLObject
        if code in tlobjects.keys():
            return self.handle_update(msg_id, sequence, reader)

        print('Unknown message: {}'.format(hex(code)))
        return False

    # endregion

    # region Message handling

    def handle_update(self, msg_id, sequence, reader):
        tlobject = reader.tgread_object()
        for handler in self.on_update_handlers:
            handler(tlobject)

        return False

    def handle_container(self, msg_id, sequence, reader, request):
        code = reader.read_int(signed=False)
        size = reader.read_int()
        for _ in range(size):
            inner_msg_id = reader.read_long(signed=False)
            inner_sequence = reader.read_int()
            inner_length = reader.read_int()
            begin_position = reader.tell_position()

            if not self.process_msg(inner_msg_id, sequence, reader, request):
                reader.set_position(begin_position + inner_length)

        return False

    def handle_bad_server_salt(self, msg_id, sequence, reader, request):
        code = reader.read_int(signed=False)
        bad_msg_id = reader.read_long(signed=False)
        bad_msg_seq_no = reader.read_int()
        error_code = reader.read_int()
        new_salt = reader.read_long(signed=False)

        self.session.salt = new_salt

        if request is None:
            raise ValueError('Tried to handle a bad server salt with no request specified')

        # Resend
        self.send(request)

        return True

    def handle_bad_msg_notification(self, msg_id, sequence, reader):
        code = reader.read_int(signed=False)
        request_id = reader.read_long(signed=False)
        request_sequence = reader.read_int()

        error_code = reader.read_int()
        raise BadMessageError(error_code)

    def handle_rpc_result(self, msg_id, sequence, reader, request):
        code = reader.read_int(signed=False)
        request_id = reader.read_long(signed=False)
        inner_code = reader.read_int(signed=False)

        if request and request_id == request.msg_id:
            request.confirm_received = True

        if inner_code == 0x2144ca19:  # RPC Error
            error = RPCError(code=reader.read_int(), message=reader.tgread_string())
            if error.must_resend:
                if not request:
                    raise ValueError('The previously sent request must be resent. '
                                     'However, no request was previously sent (called from updates thread).')
                request.confirm_received = False

            if error.message.startswith('FLOOD_WAIT_'):
                print('Should wait {}s. Sleeping until then.'.format(error.additional_data))
                sleep(error.additional_data)

            elif error.message.startswith('PHONE_MIGRATE_'):
                raise InvalidDCError(error.additional_data)

            else:
                raise error
        else:
            if not request:
                raise ValueError('Cannot receive a request from inside an RPC result from the updates thread.')

            if inner_code == 0x3072cfa1:  # GZip packed
                unpacked_data = gzip.decompress(reader.tgread_bytes())
                with BinaryReader(unpacked_data) as compressed_reader:
                    request.on_response(compressed_reader)
            else:
                reader.seek(-4)
                request.on_response(reader)

    def handle_gzip_packed(self, msg_id, sequence, reader, request):
        code = reader.read_int(signed=False)
        packed_data = reader.tgread_bytes()
        unpacked_data = gzip.decompress(packed_data)

        with BinaryReader(unpacked_data) as compressed_reader:
            return self.process_msg(msg_id, sequence, compressed_reader, request)

    # endregion

    def set_listen_for_updates(self, enabled):
        if enabled:
            if not self.updates_thread_running:
                self.updates_thread_running = True
                self.updates_thread_receiving = False

                self.updates_thread.start()
        else:
            self.updates_thread_running = False
            if self.updates_thread_receiving:
                self.transport.cancel_receive()

    def updates_thread_method(self):
        """This method will run until specified and listen for incoming updates"""

        # Set a reasonable timeout when checking for updates
        timeout = timedelta(minutes=1)

        while self.updates_thread_running:
            # Only try to receive updates if we're not waiting to receive a request
            if not self.waiting_receive:
                with self.lock:
                    try:
                        self.updates_thread_receiving = True
                        seq, body = self.transport.receive(timeout)
                        message, remote_msg_id, remote_sequence = self.decode_msg(body)

                        with BinaryReader(message) as reader:
                            self.process_msg(remote_msg_id, remote_sequence, reader)

                    except (ReadCancelledError, TimeoutError):
                        pass

                self.updates_thread_receiving = False

            # If we are here, it is because the read was cancelled
            # Sleep a bit just to give enough time for the other thread
            # to acquire the lock. No need to sleep if we're not running anymore
            if self.updates_thread_running:
                sleep(0.1)



from telethon.tl.functions.req_pq import ReqPqRequest
from telethon.tl.functions.req_dh_params import ReqDHParamsRequest
from telethon.tl.functions.set_client_dh_params import SetClientDHParamsRequest
from telethon.tl.functions.rpc_drop_answer import RpcDropAnswerRequest
from telethon.tl.functions.get_future_salts import GetFutureSaltsRequest
from telethon.tl.functions.ping import PingRequest
from telethon.tl.functions.ping_delay_disconnect import PingDelayDisconnectRequest
from telethon.tl.functions.destroy_session import DestroySessionRequest
from telethon.tl.functions.invoke_after_msg import InvokeAfterMsgRequest
from telethon.tl.functions.invoke_after_msgs import InvokeAfterMsgsRequest
from telethon.tl.functions.init_connection import InitConnectionRequest
from telethon.tl.functions.invoke_with_layer import InvokeWithLayerRequest
from telethon.tl.functions.invoke_without_updates import InvokeWithoutUpdatesRequest



from telethon.tl.mtproto_request import MTProtoRequest


class GetAllStickersRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.getAllStickers#1c9618b1 hash:int = messages.AllStickers"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x1c9618b1

    def __init__(self, hash):
        """
        :param hash: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.hash = hash

    def on_send(self, writer):
        writer.write_int(GetAllStickersRequest.constructor_id, signed=False)
        writer.write_int(self.hash)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetAllStickersRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.getAllStickers#1c9618b1 hash:int = messages.AllStickers'

    def __str__(self):
        return '(messages.getAllStickers (ID: 0x1c9618b1) = (hash={}))'.format(str(self.hash))



from telethon.tl.mtproto_request import MTProtoRequest


class InvokeAfterMsgRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    invokeAfterMsg#cb9f372d {X:Type} msg_id:long query:!X = X"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xcb9f372d

    def __init__(self, msg_id, query):
        """
        :param msg_id: Telegram type: «long».
        :param query: Telegram type: «X». This should be another MTProtoRequest.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.msg_id = msg_id
        self.query = query

    def on_send(self, writer):
        writer.write_int(InvokeAfterMsgRequest.constructor_id, signed=False)
        writer.write_long(self.msg_id)
        self.query.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InvokeAfterMsgRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'invokeAfterMsg#cb9f372d {X:Type} msg_id:long query:!X = X'

    def __str__(self):
        return '(invokeAfterMsg (ID: 0xcb9f372d) = (msg_id={}, query={}))'.format(str(self.msg_id), str(self.query))



from telethon.tl.functions.messages.get_messages import GetMessagesRequest
from telethon.tl.functions.messages.get_dialogs import GetDialogsRequest
from telethon.tl.functions.messages.get_history import GetHistoryRequest
from telethon.tl.functions.messages.search import SearchRequest
from telethon.tl.functions.messages.read_history import ReadHistoryRequest
from telethon.tl.functions.messages.delete_history import DeleteHistoryRequest
from telethon.tl.functions.messages.delete_messages import DeleteMessagesRequest
from telethon.tl.functions.messages.received_messages import ReceivedMessagesRequest
from telethon.tl.functions.messages.set_typing import SetTypingRequest
from telethon.tl.functions.messages.send_message import SendMessageRequest
from telethon.tl.functions.messages.send_media import SendMediaRequest
from telethon.tl.functions.messages.forward_messages import ForwardMessagesRequest
from telethon.tl.functions.messages.report_spam import ReportSpamRequest
from telethon.tl.functions.messages.hide_report_spam import HideReportSpamRequest
from telethon.tl.functions.messages.get_peer_settings import GetPeerSettingsRequest
from telethon.tl.functions.messages.get_chats import GetChatsRequest
from telethon.tl.functions.messages.get_full_chat import GetFullChatRequest
from telethon.tl.functions.messages.edit_chat_title import EditChatTitleRequest
from telethon.tl.functions.messages.edit_chat_photo import EditChatPhotoRequest
from telethon.tl.functions.messages.add_chat_user import AddChatUserRequest
from telethon.tl.functions.messages.delete_chat_user import DeleteChatUserRequest
from telethon.tl.functions.messages.create_chat import CreateChatRequest
from telethon.tl.functions.messages.forward_message import ForwardMessageRequest
from telethon.tl.functions.messages.get_dh_config import GetDhConfigRequest
from telethon.tl.functions.messages.request_encryption import RequestEncryptionRequest
from telethon.tl.functions.messages.accept_encryption import AcceptEncryptionRequest
from telethon.tl.functions.messages.discard_encryption import DiscardEncryptionRequest
from telethon.tl.functions.messages.set_encrypted_typing import SetEncryptedTypingRequest
from telethon.tl.functions.messages.read_encrypted_history import ReadEncryptedHistoryRequest
from telethon.tl.functions.messages.send_encrypted import SendEncryptedRequest
from telethon.tl.functions.messages.send_encrypted_file import SendEncryptedFileRequest
from telethon.tl.functions.messages.send_encrypted_service import SendEncryptedServiceRequest
from telethon.tl.functions.messages.received_queue import ReceivedQueueRequest
from telethon.tl.functions.messages.read_message_contents import ReadMessageContentsRequest
from telethon.tl.functions.messages.get_all_stickers import GetAllStickersRequest
from telethon.tl.functions.messages.get_web_page_preview import GetWebPagePreviewRequest
from telethon.tl.functions.messages.export_chat_invite import ExportChatInviteRequest
from telethon.tl.functions.messages.check_chat_invite import CheckChatInviteRequest
from telethon.tl.functions.messages.import_chat_invite import ImportChatInviteRequest
from telethon.tl.functions.messages.get_sticker_set import GetStickerSetRequest
from telethon.tl.functions.messages.install_sticker_set import InstallStickerSetRequest
from telethon.tl.functions.messages.uninstall_sticker_set import UninstallStickerSetRequest
from telethon.tl.functions.messages.start_bot import StartBotRequest
from telethon.tl.functions.messages.get_messages_views import GetMessagesViewsRequest
from telethon.tl.functions.messages.toggle_chat_admins import ToggleChatAdminsRequest
from telethon.tl.functions.messages.edit_chat_admin import EditChatAdminRequest
from telethon.tl.functions.messages.migrate_chat import MigrateChatRequest
from telethon.tl.functions.messages.search_global import SearchGlobalRequest
from telethon.tl.functions.messages.reorder_sticker_sets import ReorderStickerSetsRequest
from telethon.tl.functions.messages.get_document_by_hash import GetDocumentByHashRequest
from telethon.tl.functions.messages.search_gifs import SearchGifsRequest
from telethon.tl.functions.messages.get_saved_gifs import GetSavedGifsRequest
from telethon.tl.functions.messages.save_gif import SaveGifRequest
from telethon.tl.functions.messages.get_inline_bot_results import GetInlineBotResultsRequest
from telethon.tl.functions.messages.set_inline_bot_results import SetInlineBotResultsRequest
from telethon.tl.functions.messages.send_inline_bot_result import SendInlineBotResultRequest
from telethon.tl.functions.messages.get_message_edit_data import GetMessageEditDataRequest
from telethon.tl.functions.messages.edit_message import EditMessageRequest
from telethon.tl.functions.messages.edit_inline_bot_message import EditInlineBotMessageRequest
from telethon.tl.functions.messages.get_bot_callback_answer import GetBotCallbackAnswerRequest
from telethon.tl.functions.messages.set_bot_callback_answer import SetBotCallbackAnswerRequest
from telethon.tl.functions.messages.get_peer_dialogs import GetPeerDialogsRequest
from telethon.tl.functions.messages.save_draft import SaveDraftRequest
from telethon.tl.functions.messages.get_all_drafts import GetAllDraftsRequest
from telethon.tl.functions.messages.get_featured_stickers import GetFeaturedStickersRequest
from telethon.tl.functions.messages.read_featured_stickers import ReadFeaturedStickersRequest
from telethon.tl.functions.messages.get_recent_stickers import GetRecentStickersRequest
from telethon.tl.functions.messages.save_recent_sticker import SaveRecentStickerRequest
from telethon.tl.functions.messages.clear_recent_stickers import ClearRecentStickersRequest
from telethon.tl.functions.messages.get_archived_stickers import GetArchivedStickersRequest
from telethon.tl.functions.messages.get_mask_stickers import GetMaskStickersRequest
from telethon.tl.functions.messages.get_attached_stickers import GetAttachedStickersRequest
from telethon.tl.functions.messages.set_game_score import SetGameScoreRequest
from telethon.tl.functions.messages.set_inline_game_score import SetInlineGameScoreRequest
from telethon.tl.functions.messages.get_game_high_scores import GetGameHighScoresRequest
from telethon.tl.functions.messages.get_inline_game_high_scores import GetInlineGameHighScoresRequest



from telethon.tl.mtproto_request import MTProtoRequest


class MsgsStateInfo(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    msgs_state_info#04deb57d req_msg_id:long info:string = MsgsStateInfo"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x4deb57d

    def __init__(self, req_msg_id, info):
        """
        :param req_msg_id: Telegram type: «long».
        :param info: Telegram type: «string».
        """
        super().__init__()

        self.req_msg_id = req_msg_id
        self.info = info

    def on_send(self, writer):
        writer.write_int(MsgsStateInfo.constructor_id, signed=False)
        writer.write_long(self.req_msg_id)
        writer.tgwrite_string(self.info)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MsgsStateInfo(None, None)

    def on_response(self, reader):
        self.req_msg_id = reader.read_long()
        self.info = reader.tgread_string()

    def __repr__(self):
        return 'msgs_state_info#04deb57d req_msg_id:long info:string = MsgsStateInfo'

    def __str__(self):
        return '(msgs_state_info (ID: 0x4deb57d) = (req_msg_id={}, info={}))'.format(str(self.req_msg_id), str(self.info))



from telethon.tl.mtproto_request import MTProtoRequest


class DocumentAttributeSticker(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    documentAttributeSticker#6319d612 flags:None mask:flags.1?true alt:string stickerset:InputStickerSet mask_coords:flags.0?MaskCoords = DocumentAttribute"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6319d612

    def __init__(self, alt, stickerset, mask=None, mask_coords=None):
        """
        :param mask: Telegram type: «true».
        :param alt: Telegram type: «string».
        :param stickerset: Telegram type: «InputStickerSet».
        :param mask_coords: Telegram type: «MaskCoords».
        """
        super().__init__()

        self.mask = mask
        self.alt = alt
        self.stickerset = stickerset
        self.mask_coords = mask_coords

    def on_send(self, writer):
        writer.write_int(DocumentAttributeSticker.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 1) if self.mask else 0
        flags |= (1 << 0) if self.mask_coords else 0
        writer.write_int(flags)

        writer.tgwrite_string(self.alt)
        self.stickerset.on_send(writer)
        if self.mask_coords:
            self.mask_coords.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DocumentAttributeSticker(None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 1)) != 0:
            self.mask = True  # Arbitrary not-None value, no need to read since it is a flag

        self.alt = reader.tgread_string()
        self.stickerset = reader.tgread_object()
        if (flags & (1 << 0)) != 0:
            self.mask_coords = reader.tgread_object()

    def __repr__(self):
        return 'documentAttributeSticker#6319d612 flags:None mask:flags.1?true alt:string stickerset:InputStickerSet mask_coords:flags.0?MaskCoords = DocumentAttribute'

    def __str__(self):
        return '(documentAttributeSticker (ID: 0x6319d612) = (mask={}, alt={}, stickerset={}, mask_coords={}))'.format(str(self.mask), str(self.alt), str(self.stickerset), str(self.mask_coords))



from telethon.tl.mtproto_request import MTProtoRequest


class PingDelayDisconnectRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    ping_delay_disconnect#f3427b8c ping_id:long disconnect_delay:int = Pong"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf3427b8c

    def __init__(self, ping_id, disconnect_delay):
        """
        :param ping_id: Telegram type: «long».
        :param disconnect_delay: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.ping_id = ping_id
        self.disconnect_delay = disconnect_delay

    def on_send(self, writer):
        writer.write_int(PingDelayDisconnectRequest.constructor_id, signed=False)
        writer.write_long(self.ping_id)
        writer.write_int(self.disconnect_delay)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PingDelayDisconnectRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'ping_delay_disconnect#f3427b8c ping_id:long disconnect_delay:int = Pong'

    def __str__(self):
        return '(ping_delay_disconnect (ID: 0xf3427b8c) = (ping_id={}, disconnect_delay={}))'.format(str(self.ping_id), str(self.disconnect_delay))



from telethon.tl.mtproto_request import MTProtoRequest


class File(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    upload.file#096a18d5 type:storage.FileType mtime:int bytes:bytes = upload.File"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x96a18d5

    def __init__(self, type, mtime, bytes):
        """
        :param type: Telegram type: «storage.FileType».
        :param mtime: Telegram type: «int».
        :param bytes: Telegram type: «bytes».
        """
        super().__init__()

        self.type = type
        self.mtime = mtime
        self.bytes = bytes

    def on_send(self, writer):
        writer.write_int(File.constructor_id, signed=False)
        self.type.on_send(writer)
        writer.write_int(self.mtime)
        writer.tgwrite_bytes(self.bytes)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return File(None, None, None)

    def on_response(self, reader):
        self.type = reader.tgread_object()
        self.mtime = reader.read_int()
        self.bytes = reader.tgread_bytes()

    def __repr__(self):
        return 'upload.file#096a18d5 type:storage.FileType mtime:int bytes:bytes = upload.File'

    def __str__(self):
        return '(upload.file (ID: 0x96a18d5) = (type={}, mtime={}, bytes={}))'.format(str(self.type), str(self.mtime), str(self.bytes))



from telethon.tl.mtproto_request import MTProtoRequest


class DhGenOk(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    dh_gen_ok#3bcbf734 nonce:int128 server_nonce:int128 new_nonce_hash1:int128 = Set_client_DH_params_answer"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3bcbf734

    def __init__(self, nonce, server_nonce, new_nonce_hash1):
        """
        :param nonce: Telegram type: «int128».
        :param server_nonce: Telegram type: «int128».
        :param new_nonce_hash1: Telegram type: «int128».
        """
        super().__init__()

        self.nonce = nonce
        self.server_nonce = server_nonce
        self.new_nonce_hash1 = new_nonce_hash1

    def on_send(self, writer):
        writer.write_int(DhGenOk.constructor_id, signed=False)
        writer.write_large_int(self.nonce, bits=128)
        writer.write_large_int(self.server_nonce, bits=128)
        writer.write_large_int(self.new_nonce_hash1, bits=128)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DhGenOk(None, None, None)

    def on_response(self, reader):
        self.nonce = reader.read_large_int(bits=128)
        self.server_nonce = reader.read_large_int(bits=128)
        self.new_nonce_hash1 = reader.read_large_int(bits=128)

    def __repr__(self):
        return 'dh_gen_ok#3bcbf734 nonce:int128 server_nonce:int128 new_nonce_hash1:int128 = Set_client_DH_params_answer'

    def __str__(self):
        return '(dh_gen_ok (ID: 0x3bcbf734) = (nonce={}, server_nonce={}, new_nonce_hash1={}))'.format(str(self.nonce), str(self.server_nonce), str(self.new_nonce_hash1))



import os
import hashlib

# region Multiple utilities


def generate_random_long(signed=True):
    """Generates a random long integer (8 bytes), which is optionally signed"""
    return int.from_bytes(os.urandom(8), signed=signed, byteorder='little')


def ensure_parent_dir_exists(file_path):
    """Ensures that the parent directory exists"""
    parent = os.path.dirname(file_path)
    if parent:
        os.makedirs(parent, exist_ok=True)


# endregion

# region Cryptographic related utils


def calc_key(shared_key, msg_key, client):
    """Calculate the key based on Telegram guidelines, specifying whether it's the client or not"""
    x = 0 if client else 8

    sha1a = sha1(msg_key + shared_key[x:x + 32])
    sha1b = sha1(shared_key[x + 32:x + 48] + msg_key + shared_key[x + 48:x + 64])
    sha1c = sha1(shared_key[x + 64:x + 96] + msg_key)
    sha1d = sha1(msg_key + shared_key[x + 96:x + 128])

    key = sha1a[0:8] + sha1b[8:20] + sha1c[4:16]
    iv = sha1a[8:20] + sha1b[0:8] + sha1c[16:20] + sha1d[0:8]

    return key, iv


def calc_msg_key(data):
    """Calculates the message key from the given data"""
    return sha1(data)[4:20]


def generate_key_data_from_nonces(server_nonce, new_nonce):
    """Generates the key data corresponding to the given nonces"""
    hash1 = sha1(bytes(new_nonce + server_nonce))
    hash2 = sha1(bytes(server_nonce + new_nonce))
    hash3 = sha1(bytes(new_nonce + new_nonce))

    key = hash1 + hash2[:12]
    iv = hash2[12:20] + hash3 + new_nonce[:4]
    return key, iv


def sha1(data):
    """Calculates the SHA1 digest for the given data"""
    sha = hashlib.sha1()
    sha.update(data)
    return sha.digest()

# endregion



from telethon.tl.mtproto_request import MTProtoRequest


class BlockedSlice(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.blockedSlice#900802a1 count:int blocked:Vector<ContactBlocked> users:Vector<User> = contacts.Blocked"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x900802a1

    def __init__(self, count, blocked, users):
        """
        :param count: Telegram type: «int».
        :param blocked: Telegram type: «ContactBlocked». Must be a list.
        :param users: Telegram type: «User». Must be a list.
        """
        super().__init__()

        self.count = count
        self.blocked = blocked
        self.users = users

    def on_send(self, writer):
        writer.write_int(BlockedSlice.constructor_id, signed=False)
        writer.write_int(self.count)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.blocked))
        for blocked_item in self.blocked:
            blocked_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return BlockedSlice(None, None, None)

    def on_response(self, reader):
        self.count = reader.read_int()
        reader.read_int()  # Vector's constructor ID
        self.blocked = []  # Initialize an empty list
        blocked_len = reader.read_int()
        for _ in range(blocked_len):
            blocked_item = reader.tgread_object()
            self.blocked.append(blocked_item)

        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.tgread_object()
            self.users.append(users_item)

    def __repr__(self):
        return 'contacts.blockedSlice#900802a1 count:int blocked:Vector<ContactBlocked> users:Vector<User> = contacts.Blocked'

    def __str__(self):
        return '(contacts.blockedSlice (ID: 0x900802a1) = (count={}, blocked={}, users={}))'.format(str(self.count), None if not self.blocked else [str(_) for _ in self.blocked], None if not self.users else [str(_) for _ in self.users])



from telethon.tl.mtproto_request import MTProtoRequest


class GetTermsOfServiceRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    help.getTermsOfService#350170f3  = help.TermsOfService"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x350170f3

    def __init__(self):
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

    def on_send(self, writer):
        writer.write_int(GetTermsOfServiceRequest.constructor_id, signed=False)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetTermsOfServiceRequest()

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'help.getTermsOfService#350170f3  = help.TermsOfService'

    def __str__(self):
        return '(help.getTermsOfService (ID: 0x350170f3) = ())'.format()



from telethon.tl.mtproto_request import MTProtoRequest


class ChannelRoleModerator(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channelRoleModerator#9618d975  = ChannelParticipantRole"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9618d975

    def __init__(self):
        super().__init__()

    def on_send(self, writer):
        writer.write_int(ChannelRoleModerator.constructor_id, signed=False)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChannelRoleModerator()

    def on_response(self, reader):
        pass

    def __repr__(self):
        return 'channelRoleModerator#9618d975  = ChannelParticipantRole'

    def __str__(self):
        return '(channelRoleModerator (ID: 0x9618d975) = ())'.format()



from telethon.tl.mtproto_request import MTProtoRequest


class UninstallStickerSetRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.uninstallStickerSet#f96e55de stickerset:InputStickerSet = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf96e55de

    def __init__(self, stickerset):
        """
        :param stickerset: Telegram type: «InputStickerSet».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.stickerset = stickerset

    def on_send(self, writer):
        writer.write_int(UninstallStickerSetRequest.constructor_id, signed=False)
        self.stickerset.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UninstallStickerSetRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.uninstallStickerSet#f96e55de stickerset:InputStickerSet = Bool'

    def __str__(self):
        return '(messages.uninstallStickerSet (ID: 0xf96e55de) = (stickerset={}))'.format(str(self.stickerset))



from telethon.tl.mtproto_request import MTProtoRequest


class AppUpdate(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    help.appUpdate#8987f311 id:int critical:Bool url:string text:string = help.AppUpdate"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x8987f311

    def __init__(self, id, critical, url, text):
        """
        :param id: Telegram type: «int».
        :param critical: Telegram type: «Bool».
        :param url: Telegram type: «string».
        :param text: Telegram type: «string».
        """
        super().__init__()

        self.id = id
        self.critical = critical
        self.url = url
        self.text = text

    def on_send(self, writer):
        writer.write_int(AppUpdate.constructor_id, signed=False)
        writer.write_int(self.id)
        writer.tgwrite_bool(self.critical)
        writer.tgwrite_string(self.url)
        writer.tgwrite_string(self.text)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return AppUpdate(None, None, None, None)

    def on_response(self, reader):
        self.id = reader.read_int()
        self.critical = reader.tgread_bool()
        self.url = reader.tgread_string()
        self.text = reader.tgread_string()

    def __repr__(self):
        return 'help.appUpdate#8987f311 id:int critical:Bool url:string text:string = help.AppUpdate'

    def __str__(self):
        return '(help.appUpdate (ID: 0x8987f311) = (id={}, critical={}, url={}, text={}))'.format(str(self.id), str(self.critical), str(self.url), str(self.text))



from telethon.tl.mtproto_request import MTProtoRequest


class ChannelMessagesFilter(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channelMessagesFilter#cd77d957 flags:None exclude_new_messages:flags.1?true ranges:Vector<MessageRange> = ChannelMessagesFilter"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xcd77d957

    def __init__(self, ranges, exclude_new_messages=None):
        """
        :param exclude_new_messages: Telegram type: «true».
        :param ranges: Telegram type: «MessageRange». Must be a list.
        """
        super().__init__()

        self.exclude_new_messages = exclude_new_messages
        self.ranges = ranges

    def on_send(self, writer):
        writer.write_int(ChannelMessagesFilter.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 1) if self.exclude_new_messages else 0
        writer.write_int(flags)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.ranges))
        for ranges_item in self.ranges:
            ranges_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChannelMessagesFilter(None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 1)) != 0:
            self.exclude_new_messages = True  # Arbitrary not-None value, no need to read since it is a flag

        reader.read_int()  # Vector's constructor ID
        self.ranges = []  # Initialize an empty list
        ranges_len = reader.read_int()
        for _ in range(ranges_len):
            ranges_item = reader.tgread_object()
            self.ranges.append(ranges_item)

    def __repr__(self):
        return 'channelMessagesFilter#cd77d957 flags:None exclude_new_messages:flags.1?true ranges:Vector<MessageRange> = ChannelMessagesFilter'

    def __str__(self):
        return '(channelMessagesFilter (ID: 0xcd77d957) = (exclude_new_messages={}, ranges={}))'.format(str(self.exclude_new_messages), None if not self.ranges else [str(_) for _ in self.ranges])



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateNewMessage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateNewMessage#1f2b0afd message:Message pts:int pts_count:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x1f2b0afd

    def __init__(self, message, pts, pts_count):
        """
        :param message: Telegram type: «Message».
        :param pts: Telegram type: «int».
        :param pts_count: Telegram type: «int».
        """
        super().__init__()

        self.message = message
        self.pts = pts
        self.pts_count = pts_count

    def on_send(self, writer):
        writer.write_int(UpdateNewMessage.constructor_id, signed=False)
        self.message.on_send(writer)
        writer.write_int(self.pts)
        writer.write_int(self.pts_count)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateNewMessage(None, None, None)

    def on_response(self, reader):
        self.message = reader.tgread_object()
        self.pts = reader.read_int()
        self.pts_count = reader.read_int()

    def __repr__(self):
        return 'updateNewMessage#1f2b0afd message:Message pts:int pts_count:int = Update'

    def __str__(self):
        return '(updateNewMessage (ID: 0x1f2b0afd) = (message={}, pts={}, pts_count={}))'.format(str(self.message), str(self.pts), str(self.pts_count))



from telethon.tl.mtproto_request import MTProtoRequest


class Document(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    document#87232bc7 id:long access_hash:long date:date mime_type:string size:int thumb:PhotoSize dc_id:int version:int attributes:Vector<DocumentAttribute> = Document"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x87232bc7

    def __init__(self, id, access_hash, date, mime_type, size, thumb, dc_id, version, attributes):
        """
        :param id: Telegram type: «long».
        :param access_hash: Telegram type: «long».
        :param date: Telegram type: «date».
        :param mime_type: Telegram type: «string».
        :param size: Telegram type: «int».
        :param thumb: Telegram type: «PhotoSize».
        :param dc_id: Telegram type: «int».
        :param version: Telegram type: «int».
        :param attributes: Telegram type: «DocumentAttribute». Must be a list.
        """
        super().__init__()

        self.id = id
        self.access_hash = access_hash
        self.date = date
        self.mime_type = mime_type
        self.size = size
        self.thumb = thumb
        self.dc_id = dc_id
        self.version = version
        self.attributes = attributes

    def on_send(self, writer):
        writer.write_int(Document.constructor_id, signed=False)
        writer.write_long(self.id)
        writer.write_long(self.access_hash)
        writer.tgwrite_date(self.date)
        writer.tgwrite_string(self.mime_type)
        writer.write_int(self.size)
        self.thumb.on_send(writer)
        writer.write_int(self.dc_id)
        writer.write_int(self.version)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.attributes))
        for attributes_item in self.attributes:
            attributes_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Document(None, None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        self.id = reader.read_long()
        self.access_hash = reader.read_long()
        self.date = reader.tgread_date()
        self.mime_type = reader.tgread_string()
        self.size = reader.read_int()
        self.thumb = reader.tgread_object()
        self.dc_id = reader.read_int()
        self.version = reader.read_int()
        reader.read_int()  # Vector's constructor ID
        self.attributes = []  # Initialize an empty list
        attributes_len = reader.read_int()
        for _ in range(attributes_len):
            attributes_item = reader.tgread_object()
            self.attributes.append(attributes_item)

    def __repr__(self):
        return 'document#87232bc7 id:long access_hash:long date:date mime_type:string size:int thumb:PhotoSize dc_id:int version:int attributes:Vector<DocumentAttribute> = Document'

    def __str__(self):
        return '(document (ID: 0x87232bc7) = (id={}, access_hash={}, date={}, mime_type={}, size={}, thumb={}, dc_id={}, version={}, attributes={}))'.format(str(self.id), str(self.access_hash), str(self.date), str(self.mime_type), str(self.size), str(self.thumb), str(self.dc_id), str(self.version), None if not self.attributes else [str(_) for _ in self.attributes])



from telethon.tl.mtproto_request import MTProtoRequest


class BotInfo(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    botInfo#98e81d3a user_id:int description:string commands:Vector<BotCommand> = BotInfo"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x98e81d3a

    def __init__(self, user_id, description, commands):
        """
        :param user_id: Telegram type: «int».
        :param description: Telegram type: «string».
        :param commands: Telegram type: «BotCommand». Must be a list.
        """
        super().__init__()

        self.user_id = user_id
        self.description = description
        self.commands = commands

    def on_send(self, writer):
        writer.write_int(BotInfo.constructor_id, signed=False)
        writer.write_int(self.user_id)
        writer.tgwrite_string(self.description)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.commands))
        for commands_item in self.commands:
            commands_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return BotInfo(None, None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.description = reader.tgread_string()
        reader.read_int()  # Vector's constructor ID
        self.commands = []  # Initialize an empty list
        commands_len = reader.read_int()
        for _ in range(commands_len):
            commands_item = reader.tgread_object()
            self.commands.append(commands_item)

    def __repr__(self):
        return 'botInfo#98e81d3a user_id:int description:string commands:Vector<BotCommand> = BotInfo'

    def __str__(self):
        return '(botInfo (ID: 0x98e81d3a) = (user_id={}, description={}, commands={}))'.format(str(self.user_id), str(self.description), None if not self.commands else [str(_) for _ in self.commands])



from telethon.tl.mtproto_request import MTProtoRequest


class Contacts(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.contacts#6f8b8cb2 contacts:Vector<Contact> users:Vector<User> = contacts.Contacts"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6f8b8cb2

    def __init__(self, contacts, users):
        """
        :param contacts: Telegram type: «Contact». Must be a list.
        :param users: Telegram type: «User». Must be a list.
        """
        super().__init__()

        self.contacts = contacts
        self.users = users

    def on_send(self, writer):
        writer.write_int(Contacts.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.contacts))
        for contacts_item in self.contacts:
            contacts_item.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Contacts(None, None)

    def on_response(self, reader):
        reader.read_int()  # Vector's constructor ID
        self.contacts = []  # Initialize an empty list
        contacts_len = reader.read_int()
        for _ in range(contacts_len):
            contacts_item = reader.tgread_object()
            self.contacts.append(contacts_item)

        reader.read_int()  # Vector's constructor ID
        self.users = []  # Initialize an empty list
        users_len = reader.read_int()
        for _ in range(users_len):
            users_item = reader.tgread_object()
            self.users.append(users_item)

    def __repr__(self):
        return 'contacts.contacts#6f8b8cb2 contacts:Vector<Contact> users:Vector<User> = contacts.Contacts'

    def __str__(self):
        return '(contacts.contacts (ID: 0x6f8b8cb2) = (contacts={}, users={}))'.format(None if not self.contacts else [str(_) for _ in self.contacts], None if not self.users else [str(_) for _ in self.users])



from telethon.tl.mtproto_request import MTProtoRequest


class InputPeerChat(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputPeerChat#179be863 chat_id:int = InputPeer"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x179be863

    def __init__(self, chat_id):
        """
        :param chat_id: Telegram type: «int».
        """
        super().__init__()

        self.chat_id = chat_id

    def on_send(self, writer):
        writer.write_int(InputPeerChat.constructor_id, signed=False)
        writer.write_int(self.chat_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputPeerChat(None)

    def on_response(self, reader):
        self.chat_id = reader.read_int()

    def __repr__(self):
        return 'inputPeerChat#179be863 chat_id:int = InputPeer'

    def __str__(self):
        return '(inputPeerChat (ID: 0x179be863) = (chat_id={}))'.format(str(self.chat_id))



from telethon.tl.functions.channels.read_history import ReadHistoryRequest
from telethon.tl.functions.channels.delete_messages import DeleteMessagesRequest
from telethon.tl.functions.channels.delete_user_history import DeleteUserHistoryRequest
from telethon.tl.functions.channels.report_spam import ReportSpamRequest
from telethon.tl.functions.channels.get_messages import GetMessagesRequest
from telethon.tl.functions.channels.get_participants import GetParticipantsRequest
from telethon.tl.functions.channels.get_participant import GetParticipantRequest
from telethon.tl.functions.channels.get_channels import GetChannelsRequest
from telethon.tl.functions.channels.get_full_channel import GetFullChannelRequest
from telethon.tl.functions.channels.create_channel import CreateChannelRequest
from telethon.tl.functions.channels.edit_about import EditAboutRequest
from telethon.tl.functions.channels.edit_admin import EditAdminRequest
from telethon.tl.functions.channels.edit_title import EditTitleRequest
from telethon.tl.functions.channels.edit_photo import EditPhotoRequest
from telethon.tl.functions.channels.check_username import CheckUsernameRequest
from telethon.tl.functions.channels.update_username import UpdateUsernameRequest
from telethon.tl.functions.channels.join_channel import JoinChannelRequest
from telethon.tl.functions.channels.leave_channel import LeaveChannelRequest
from telethon.tl.functions.channels.invite_to_channel import InviteToChannelRequest
from telethon.tl.functions.channels.kick_from_channel import KickFromChannelRequest
from telethon.tl.functions.channels.export_invite import ExportInviteRequest
from telethon.tl.functions.channels.delete_channel import DeleteChannelRequest
from telethon.tl.functions.channels.toggle_invites import ToggleInvitesRequest
from telethon.tl.functions.channels.export_message_link import ExportMessageLinkRequest
from telethon.tl.functions.channels.toggle_signatures import ToggleSignaturesRequest
from telethon.tl.functions.channels.update_pinned_message import UpdatePinnedMessageRequest
from telethon.tl.functions.channels.get_admined_public_channels import GetAdminedPublicChannelsRequest



from telethon.tl.mtproto_request import MTProtoRequest


class UpdatePasswordSettingsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.updatePasswordSettings#fa7c4b86 current_password_hash:bytes new_settings:account.PasswordInputSettings = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xfa7c4b86

    def __init__(self, current_password_hash, new_settings):
        """
        :param current_password_hash: Telegram type: «bytes».
        :param new_settings: Telegram type: «account.PasswordInputSettings».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.current_password_hash = current_password_hash
        self.new_settings = new_settings

    def on_send(self, writer):
        writer.write_int(UpdatePasswordSettingsRequest.constructor_id, signed=False)
        writer.tgwrite_bytes(self.current_password_hash)
        self.new_settings.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdatePasswordSettingsRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'account.updatePasswordSettings#fa7c4b86 current_password_hash:bytes new_settings:account.PasswordInputSettings = Bool'

    def __str__(self):
        return '(account.updatePasswordSettings (ID: 0xfa7c4b86) = (current_password_hash={}, new_settings={}))'.format(str(self.current_password_hash), str(self.new_settings))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateChatAdmins(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChatAdmins#6e947941 chat_id:int enabled:Bool version:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x6e947941

    def __init__(self, chat_id, enabled, version):
        """
        :param chat_id: Telegram type: «int».
        :param enabled: Telegram type: «Bool».
        :param version: Telegram type: «int».
        """
        super().__init__()

        self.chat_id = chat_id
        self.enabled = enabled
        self.version = version

    def on_send(self, writer):
        writer.write_int(UpdateChatAdmins.constructor_id, signed=False)
        writer.write_int(self.chat_id)
        writer.tgwrite_bool(self.enabled)
        writer.write_int(self.version)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChatAdmins(None, None, None)

    def on_response(self, reader):
        self.chat_id = reader.read_int()
        self.enabled = reader.tgread_bool()
        self.version = reader.read_int()

    def __repr__(self):
        return 'updateChatAdmins#6e947941 chat_id:int enabled:Bool version:int = Update'

    def __str__(self):
        return '(updateChatAdmins (ID: 0x6e947941) = (chat_id={}, enabled={}, version={}))'.format(str(self.chat_id), str(self.enabled), str(self.version))



from telethon.tl.mtproto_request import MTProtoRequest


class ImportContactsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.importContacts#da30b32d contacts:Vector<InputContact> replace:Bool = contacts.ImportedContacts"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xda30b32d

    def __init__(self, contacts, replace):
        """
        :param contacts: Telegram type: «InputContact». Must be a list.
        :param replace: Telegram type: «Bool».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.contacts = contacts
        self.replace = replace

    def on_send(self, writer):
        writer.write_int(ImportContactsRequest.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.contacts))
        for contacts_item in self.contacts:
            contacts_item.on_send(writer)

        writer.tgwrite_bool(self.replace)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ImportContactsRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'contacts.importContacts#da30b32d contacts:Vector<InputContact> replace:Bool = contacts.ImportedContacts'

    def __str__(self):
        return '(contacts.importContacts (ID: 0xda30b32d) = (contacts={}, replace={}))'.format(None if not self.contacts else [str(_) for _ in self.contacts], str(self.replace))



from telethon.tl.mtproto_request import MTProtoRequest


class BotInlineMessageMediaGeo(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    botInlineMessageMediaGeo#3a8fd8b8 flags:None geo:GeoPoint reply_markup:flags.2?ReplyMarkup = BotInlineMessage"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3a8fd8b8

    def __init__(self, geo, reply_markup=None):
        """
        :param geo: Telegram type: «GeoPoint».
        :param reply_markup: Telegram type: «ReplyMarkup».
        """
        super().__init__()

        self.geo = geo
        self.reply_markup = reply_markup

    def on_send(self, writer):
        writer.write_int(BotInlineMessageMediaGeo.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 2) if self.reply_markup else 0
        writer.write_int(flags)

        self.geo.on_send(writer)
        if self.reply_markup:
            self.reply_markup.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return BotInlineMessageMediaGeo(None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.geo = reader.tgread_object()
        if (flags & (1 << 2)) != 0:
            self.reply_markup = reader.tgread_object()

    def __repr__(self):
        return 'botInlineMessageMediaGeo#3a8fd8b8 flags:None geo:GeoPoint reply_markup:flags.2?ReplyMarkup = BotInlineMessage'

    def __str__(self):
        return '(botInlineMessageMediaGeo (ID: 0x3a8fd8b8) = (geo={}, reply_markup={}))'.format(str(self.geo), str(self.reply_markup))



from telethon.tl.mtproto_request import MTProtoRequest


class SearchRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.search#d4569248 flags:None peer:InputPeer q:string filter:MessagesFilter min_date:date max_date:date offset:int max_id:int limit:int = messages.Messages"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xd4569248

    def __init__(self, peer, q, filter, min_date, max_date, offset, max_id, limit):
        """
        :param peer: Telegram type: «InputPeer».
        :param q: Telegram type: «string».
        :param filter: Telegram type: «MessagesFilter».
        :param min_date: Telegram type: «date».
        :param max_date: Telegram type: «date».
        :param offset: Telegram type: «int».
        :param max_id: Telegram type: «int».
        :param limit: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.peer = peer
        self.q = q
        self.filter = filter
        self.min_date = min_date
        self.max_date = max_date
        self.offset = offset
        self.max_id = max_id
        self.limit = limit

    def on_send(self, writer):
        writer.write_int(SearchRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        writer.write_int(flags)

        self.peer.on_send(writer)
        writer.tgwrite_string(self.q)
        self.filter.on_send(writer)
        writer.tgwrite_date(self.min_date)
        writer.tgwrite_date(self.max_date)
        writer.write_int(self.offset)
        writer.write_int(self.max_id)
        writer.write_int(self.limit)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SearchRequest(None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.search#d4569248 flags:None peer:InputPeer q:string filter:MessagesFilter min_date:date max_date:date offset:int max_id:int limit:int = messages.Messages'

    def __str__(self):
        return '(messages.search (ID: 0xd4569248) = (peer={}, q={}, filter={}, min_date={}, max_date={}, offset={}, max_id={}, limit={}))'.format(str(self.peer), str(self.q), str(self.filter), str(self.min_date), str(self.max_date), str(self.offset), str(self.max_id), str(self.limit))



from telethon.tl.mtproto_request import MTProtoRequest


class WallPaperSolid(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    wallPaperSolid#63117f24 id:int title:string bg_color:int color:int = WallPaper"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x63117f24

    def __init__(self, id, title, bg_color, color):
        """
        :param id: Telegram type: «int».
        :param title: Telegram type: «string».
        :param bg_color: Telegram type: «int».
        :param color: Telegram type: «int».
        """
        super().__init__()

        self.id = id
        self.title = title
        self.bg_color = bg_color
        self.color = color

    def on_send(self, writer):
        writer.write_int(WallPaperSolid.constructor_id, signed=False)
        writer.write_int(self.id)
        writer.tgwrite_string(self.title)
        writer.write_int(self.bg_color)
        writer.write_int(self.color)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return WallPaperSolid(None, None, None, None)

    def on_response(self, reader):
        self.id = reader.read_int()
        self.title = reader.tgread_string()
        self.bg_color = reader.read_int()
        self.color = reader.read_int()

    def __repr__(self):
        return 'wallPaperSolid#63117f24 id:int title:string bg_color:int color:int = WallPaper'

    def __str__(self):
        return '(wallPaperSolid (ID: 0x63117f24) = (id={}, title={}, bg_color={}, color={}))'.format(str(self.id), str(self.title), str(self.bg_color), str(self.color))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateChannelMessageViews(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChannelMessageViews#98a12b4b channel_id:int id:int views:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x98a12b4b

    def __init__(self, channel_id, id, views):
        """
        :param channel_id: Telegram type: «int».
        :param id: Telegram type: «int».
        :param views: Telegram type: «int».
        """
        super().__init__()

        self.channel_id = channel_id
        self.id = id
        self.views = views

    def on_send(self, writer):
        writer.write_int(UpdateChannelMessageViews.constructor_id, signed=False)
        writer.write_int(self.channel_id)
        writer.write_int(self.id)
        writer.write_int(self.views)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChannelMessageViews(None, None, None)

    def on_response(self, reader):
        self.channel_id = reader.read_int()
        self.id = reader.read_int()
        self.views = reader.read_int()

    def __repr__(self):
        return 'updateChannelMessageViews#98a12b4b channel_id:int id:int views:int = Update'

    def __str__(self):
        return '(updateChannelMessageViews (ID: 0x98a12b4b) = (channel_id={}, id={}, views={}))'.format(str(self.channel_id), str(self.id), str(self.views))



from telethon.tl.mtproto_request import MTProtoRequest


class ResetNotifySettingsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.resetNotifySettings#db7e1747  = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xdb7e1747

    def __init__(self):
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

    def on_send(self, writer):
        writer.write_int(ResetNotifySettingsRequest.constructor_id, signed=False)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ResetNotifySettingsRequest()

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'account.resetNotifySettings#db7e1747  = Bool'

    def __str__(self):
        return '(account.resetNotifySettings (ID: 0xdb7e1747) = ())'.format()



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateUserBlocked(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateUserBlocked#80ece81a user_id:int blocked:Bool = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x80ece81a

    def __init__(self, user_id, blocked):
        """
        :param user_id: Telegram type: «int».
        :param blocked: Telegram type: «Bool».
        """
        super().__init__()

        self.user_id = user_id
        self.blocked = blocked

    def on_send(self, writer):
        writer.write_int(UpdateUserBlocked.constructor_id, signed=False)
        writer.write_int(self.user_id)
        writer.tgwrite_bool(self.blocked)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateUserBlocked(None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.blocked = reader.tgread_bool()

    def __repr__(self):
        return 'updateUserBlocked#80ece81a user_id:int blocked:Bool = Update'

    def __str__(self):
        return '(updateUserBlocked (ID: 0x80ece81a) = (user_id={}, blocked={}))'.format(str(self.user_id), str(self.blocked))



from telethon.tl.mtproto_request import MTProtoRequest


class PhotoSize(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    photoSize#77bfb61b type:string location:FileLocation w:int h:int size:int = PhotoSize"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x77bfb61b

    def __init__(self, type, location, w, h, size):
        """
        :param type: Telegram type: «string».
        :param location: Telegram type: «FileLocation».
        :param w: Telegram type: «int».
        :param h: Telegram type: «int».
        :param size: Telegram type: «int».
        """
        super().__init__()

        self.type = type
        self.location = location
        self.w = w
        self.h = h
        self.size = size

    def on_send(self, writer):
        writer.write_int(PhotoSize.constructor_id, signed=False)
        writer.tgwrite_string(self.type)
        self.location.on_send(writer)
        writer.write_int(self.w)
        writer.write_int(self.h)
        writer.write_int(self.size)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PhotoSize(None, None, None, None, None)

    def on_response(self, reader):
        self.type = reader.tgread_string()
        self.location = reader.tgread_object()
        self.w = reader.read_int()
        self.h = reader.read_int()
        self.size = reader.read_int()

    def __repr__(self):
        return 'photoSize#77bfb61b type:string location:FileLocation w:int h:int size:int = PhotoSize'

    def __str__(self):
        return '(photoSize (ID: 0x77bfb61b) = (type={}, location={}, w={}, h={}, size={}))'.format(str(self.type), str(self.location), str(self.w), str(self.h), str(self.size))



from telethon.tl.mtproto_request import MTProtoRequest


class ChannelParticipantEditor(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channelParticipantEditor#98192d61 user_id:int inviter_id:int date:date = ChannelParticipant"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x98192d61

    def __init__(self, user_id, inviter_id, date):
        """
        :param user_id: Telegram type: «int».
        :param inviter_id: Telegram type: «int».
        :param date: Telegram type: «date».
        """
        super().__init__()

        self.user_id = user_id
        self.inviter_id = inviter_id
        self.date = date

    def on_send(self, writer):
        writer.write_int(ChannelParticipantEditor.constructor_id, signed=False)
        writer.write_int(self.user_id)
        writer.write_int(self.inviter_id)
        writer.tgwrite_date(self.date)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChannelParticipantEditor(None, None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.inviter_id = reader.read_int()
        self.date = reader.tgread_date()

    def __repr__(self):
        return 'channelParticipantEditor#98192d61 user_id:int inviter_id:int date:date = ChannelParticipant'

    def __str__(self):
        return '(channelParticipantEditor (ID: 0x98192d61) = (user_id={}, inviter_id={}, date={}))'.format(str(self.user_id), str(self.inviter_id), str(self.date))



from telethon.tl.mtproto_request import MTProtoRequest


class DocumentAttributeVideo(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    documentAttributeVideo#5910cccb duration:int w:int h:int = DocumentAttribute"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x5910cccb

    def __init__(self, duration, w, h):
        """
        :param duration: Telegram type: «int».
        :param w: Telegram type: «int».
        :param h: Telegram type: «int».
        """
        super().__init__()

        self.duration = duration
        self.w = w
        self.h = h

    def on_send(self, writer):
        writer.write_int(DocumentAttributeVideo.constructor_id, signed=False)
        writer.write_int(self.duration)
        writer.write_int(self.w)
        writer.write_int(self.h)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DocumentAttributeVideo(None, None, None)

    def on_response(self, reader):
        self.duration = reader.read_int()
        self.w = reader.read_int()
        self.h = reader.read_int()

    def __repr__(self):
        return 'documentAttributeVideo#5910cccb duration:int w:int h:int = DocumentAttribute'

    def __str__(self):
        return '(documentAttributeVideo (ID: 0x5910cccb) = (duration={}, w={}, h={}))'.format(str(self.duration), str(self.w), str(self.h))



from telethon.tl.mtproto_request import MTProtoRequest


class AcceptEncryptionRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.acceptEncryption#3dbc0415 peer:InputEncryptedChat g_b:bytes key_fingerprint:long = EncryptedChat"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3dbc0415

    def __init__(self, peer, g_b, key_fingerprint):
        """
        :param peer: Telegram type: «InputEncryptedChat».
        :param g_b: Telegram type: «bytes».
        :param key_fingerprint: Telegram type: «long».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.peer = peer
        self.g_b = g_b
        self.key_fingerprint = key_fingerprint

    def on_send(self, writer):
        writer.write_int(AcceptEncryptionRequest.constructor_id, signed=False)
        self.peer.on_send(writer)
        writer.tgwrite_bytes(self.g_b)
        writer.write_long(self.key_fingerprint)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return AcceptEncryptionRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.acceptEncryption#3dbc0415 peer:InputEncryptedChat g_b:bytes key_fingerprint:long = EncryptedChat'

    def __str__(self):
        return '(messages.acceptEncryption (ID: 0x3dbc0415) = (peer={}, g_b={}, key_fingerprint={}))'.format(str(self.peer), str(self.g_b), str(self.key_fingerprint))



from telethon.tl.mtproto_request import MTProtoRequest


class BotResults(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.botResults#256709a6 flags:None gallery:flags.0?true query_id:long next_offset:flags.1?string switch_pm:flags.2?InlineBotSwitchPM results:Vector<BotInlineResult> = messages.BotResults"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x256709a6

    def __init__(self, query_id, results, gallery=None, next_offset=None, switch_pm=None):
        """
        :param gallery: Telegram type: «true».
        :param query_id: Telegram type: «long».
        :param next_offset: Telegram type: «string».
        :param switch_pm: Telegram type: «InlineBotSwitchPM».
        :param results: Telegram type: «BotInlineResult». Must be a list.
        """
        super().__init__()

        self.gallery = gallery
        self.query_id = query_id
        self.next_offset = next_offset
        self.switch_pm = switch_pm
        self.results = results

    def on_send(self, writer):
        writer.write_int(BotResults.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.gallery else 0
        flags |= (1 << 1) if self.next_offset else 0
        flags |= (1 << 2) if self.switch_pm else 0
        writer.write_int(flags)

        writer.write_long(self.query_id)
        if self.next_offset:
            writer.tgwrite_string(self.next_offset)

        if self.switch_pm:
            self.switch_pm.on_send(writer)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.results))
        for results_item in self.results:
            results_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return BotResults(None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.gallery = True  # Arbitrary not-None value, no need to read since it is a flag

        self.query_id = reader.read_long()
        if (flags & (1 << 1)) != 0:
            self.next_offset = reader.tgread_string()

        if (flags & (1 << 2)) != 0:
            self.switch_pm = reader.tgread_object()

        reader.read_int()  # Vector's constructor ID
        self.results = []  # Initialize an empty list
        results_len = reader.read_int()
        for _ in range(results_len):
            results_item = reader.tgread_object()
            self.results.append(results_item)

    def __repr__(self):
        return 'messages.botResults#256709a6 flags:None gallery:flags.0?true query_id:long next_offset:flags.1?string switch_pm:flags.2?InlineBotSwitchPM results:Vector<BotInlineResult> = messages.BotResults'

    def __str__(self):
        return '(messages.botResults (ID: 0x256709a6) = (gallery={}, query_id={}, next_offset={}, switch_pm={}, results={}))'.format(str(self.gallery), str(self.query_id), str(self.next_offset), str(self.switch_pm), None if not self.results else [str(_) for _ in self.results])



from telethon.tl.mtproto_request import MTProtoRequest


class BadServerSalt(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    bad_server_salt#edab447b bad_msg_id:long bad_msg_seqno:int error_code:int new_server_salt:long = BadMsgNotification"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xedab447b

    def __init__(self, bad_msg_id, bad_msg_seqno, error_code, new_server_salt):
        """
        :param bad_msg_id: Telegram type: «long».
        :param bad_msg_seqno: Telegram type: «int».
        :param error_code: Telegram type: «int».
        :param new_server_salt: Telegram type: «long».
        """
        super().__init__()

        self.bad_msg_id = bad_msg_id
        self.bad_msg_seqno = bad_msg_seqno
        self.error_code = error_code
        self.new_server_salt = new_server_salt

    def on_send(self, writer):
        writer.write_int(BadServerSalt.constructor_id, signed=False)
        writer.write_long(self.bad_msg_id)
        writer.write_int(self.bad_msg_seqno)
        writer.write_int(self.error_code)
        writer.write_long(self.new_server_salt)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return BadServerSalt(None, None, None, None)

    def on_response(self, reader):
        self.bad_msg_id = reader.read_long()
        self.bad_msg_seqno = reader.read_int()
        self.error_code = reader.read_int()
        self.new_server_salt = reader.read_long()

    def __repr__(self):
        return 'bad_server_salt#edab447b bad_msg_id:long bad_msg_seqno:int error_code:int new_server_salt:long = BadMsgNotification'

    def __str__(self):
        return '(bad_server_salt (ID: 0xedab447b) = (bad_msg_id={}, bad_msg_seqno={}, error_code={}, new_server_salt={}))'.format(str(self.bad_msg_id), str(self.bad_msg_seqno), str(self.error_code), str(self.new_server_salt))



from telethon.tl.mtproto_request import MTProtoRequest


class InviteToChannelRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channels.inviteToChannel#199f3a6c channel:InputChannel users:Vector<InputUser> = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x199f3a6c

    def __init__(self, channel, users):
        """
        :param channel: Telegram type: «InputChannel».
        :param users: Telegram type: «InputUser». Must be a list.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.channel = channel
        self.users = users

    def on_send(self, writer):
        writer.write_int(InviteToChannelRequest.constructor_id, signed=False)
        self.channel.on_send(writer)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.users))
        for users_item in self.users:
            users_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InviteToChannelRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'channels.inviteToChannel#199f3a6c channel:InputChannel users:Vector<InputUser> = Updates'

    def __str__(self):
        return '(channels.inviteToChannel (ID: 0x199f3a6c) = (channel={}, users={}))'.format(str(self.channel), None if not self.users else [str(_) for _ in self.users])



from telethon.tl.mtproto_request import MTProtoRequest


class SetPrivacyRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.setPrivacy#c9f81ce8 key:InputPrivacyKey rules:Vector<InputPrivacyRule> = account.PrivacyRules"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xc9f81ce8

    def __init__(self, key, rules):
        """
        :param key: Telegram type: «InputPrivacyKey».
        :param rules: Telegram type: «InputPrivacyRule». Must be a list.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.key = key
        self.rules = rules

    def on_send(self, writer):
        writer.write_int(SetPrivacyRequest.constructor_id, signed=False)
        self.key.on_send(writer)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.rules))
        for rules_item in self.rules:
            rules_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SetPrivacyRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'account.setPrivacy#c9f81ce8 key:InputPrivacyKey rules:Vector<InputPrivacyRule> = account.PrivacyRules'

    def __str__(self):
        return '(account.setPrivacy (ID: 0xc9f81ce8) = (key={}, rules={}))'.format(str(self.key), None if not self.rules else [str(_) for _ in self.rules])



from telethon.tl.mtproto_request import MTProtoRequest


class GetMessagesViewsRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.getMessagesViews#c4c8a55d peer:InputPeer id:Vector<int> increment:Bool = Vector<int>"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xc4c8a55d

    def __init__(self, peer, id, increment):
        """
        :param peer: Telegram type: «InputPeer».
        :param id: Telegram type: «int». Must be a list.
        :param increment: Telegram type: «Bool».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.peer = peer
        self.id = id
        self.increment = increment

    def on_send(self, writer):
        writer.write_int(GetMessagesViewsRequest.constructor_id, signed=False)
        self.peer.on_send(writer)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.id))
        for id_item in self.id:
            writer.write_int(id_item)

        writer.tgwrite_bool(self.increment)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetMessagesViewsRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.getMessagesViews#c4c8a55d peer:InputPeer id:Vector<int> increment:Bool = Vector<int>'

    def __str__(self):
        return '(messages.getMessagesViews (ID: 0xc4c8a55d) = (peer={}, id={}, increment={}))'.format(str(self.peer), None if not self.id else [str(_) for _ in self.id], str(self.increment))



from telethon.tl.mtproto_request import MTProtoRequest


class StickerSet(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    stickerSet#cd303b41 flags:None installed:flags.0?true archived:flags.1?true official:flags.2?true masks:flags.3?true id:long access_hash:long title:string short_name:string count:int hash:int = StickerSet"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xcd303b41

    def __init__(self, id, access_hash, title, short_name, count, hash, installed=None, archived=None, official=None, masks=None):
        """
        :param installed: Telegram type: «true».
        :param archived: Telegram type: «true».
        :param official: Telegram type: «true».
        :param masks: Telegram type: «true».
        :param id: Telegram type: «long».
        :param access_hash: Telegram type: «long».
        :param title: Telegram type: «string».
        :param short_name: Telegram type: «string».
        :param count: Telegram type: «int».
        :param hash: Telegram type: «int».
        """
        super().__init__()

        self.installed = installed
        self.archived = archived
        self.official = official
        self.masks = masks
        self.id = id
        self.access_hash = access_hash
        self.title = title
        self.short_name = short_name
        self.count = count
        self.hash = hash

    def on_send(self, writer):
        writer.write_int(StickerSet.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.installed else 0
        flags |= (1 << 1) if self.archived else 0
        flags |= (1 << 2) if self.official else 0
        flags |= (1 << 3) if self.masks else 0
        writer.write_int(flags)

        writer.write_long(self.id)
        writer.write_long(self.access_hash)
        writer.tgwrite_string(self.title)
        writer.tgwrite_string(self.short_name)
        writer.write_int(self.count)
        writer.write_int(self.hash)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return StickerSet(None, None, None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.installed = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 1)) != 0:
            self.archived = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 2)) != 0:
            self.official = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 3)) != 0:
            self.masks = True  # Arbitrary not-None value, no need to read since it is a flag

        self.id = reader.read_long()
        self.access_hash = reader.read_long()
        self.title = reader.tgread_string()
        self.short_name = reader.tgread_string()
        self.count = reader.read_int()
        self.hash = reader.read_int()

    def __repr__(self):
        return 'stickerSet#cd303b41 flags:None installed:flags.0?true archived:flags.1?true official:flags.2?true masks:flags.3?true id:long access_hash:long title:string short_name:string count:int hash:int = StickerSet'

    def __str__(self):
        return '(stickerSet (ID: 0xcd303b41) = (installed={}, archived={}, official={}, masks={}, id={}, access_hash={}, title={}, short_name={}, count={}, hash={}))'.format(str(self.installed), str(self.archived), str(self.official), str(self.masks), str(self.id), str(self.access_hash), str(self.title), str(self.short_name), str(self.count), str(self.hash))



from telethon.tl.mtproto_request import MTProtoRequest


class BindTempAuthKeyRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.bindTempAuthKey#cdd42a05 perm_auth_key_id:long nonce:long expires_at:int encrypted_message:bytes = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xcdd42a05

    def __init__(self, perm_auth_key_id, nonce, expires_at, encrypted_message):
        """
        :param perm_auth_key_id: Telegram type: «long».
        :param nonce: Telegram type: «long».
        :param expires_at: Telegram type: «int».
        :param encrypted_message: Telegram type: «bytes».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.perm_auth_key_id = perm_auth_key_id
        self.nonce = nonce
        self.expires_at = expires_at
        self.encrypted_message = encrypted_message

    def on_send(self, writer):
        writer.write_int(BindTempAuthKeyRequest.constructor_id, signed=False)
        writer.write_long(self.perm_auth_key_id)
        writer.write_long(self.nonce)
        writer.write_int(self.expires_at)
        writer.tgwrite_bytes(self.encrypted_message)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return BindTempAuthKeyRequest(None, None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'auth.bindTempAuthKey#cdd42a05 perm_auth_key_id:long nonce:long expires_at:int encrypted_message:bytes = Bool'

    def __str__(self):
        return '(auth.bindTempAuthKey (ID: 0xcdd42a05) = (perm_auth_key_id={}, nonce={}, expires_at={}, encrypted_message={}))'.format(str(self.perm_auth_key_id), str(self.nonce), str(self.expires_at), str(self.encrypted_message))
